"""Basic usage example for WeChat XPay SDK."""

import asyncio

from wechat_xpay import XPayAsyncClient, XPayClient
from wechat_xpay.webhook import WebhookParser


def sync_examples():
    """同步客户端使用示例。"""
    print("=" * 60)
    print("同步客户端示例")
    print("=" * 60)

    # 方式 1：上下文管理器（推荐）
    with XPayClient(
        app_id="wx1234567890",
        app_key="your_app_key",
        env=0,  # 0=沙箱，1=生产环境
    ) as client:
        # 示例 1：查询用户余额
        print("\n=== 查询用户余额 ===")
        try:
            # 注意：session_key 在调用时传入，因为它会定期过期
            balance = client.query_user_balance(
                openid="user_openid",
                session_key="user_session_key",
            )
            print(f"余额: {balance.balance}")
            print(f"赠送余额: {balance.present_balance}")
            print(f"累计充值: {balance.sum_save}")
        except Exception as e:
            print(f"错误: {e}")

        # 示例 2：处理支付（扣除代币）
        print("\n=== 处理支付 ===")
        try:
            result = client.currency_pay(
                openid="user_openid",
                session_key="user_session_key",
                order_id="ORDER_20240101_001",
                amount=100,  # 代币数量
                payitem="高级道具",
            )
            print(f"订单 ID: {result.order_id}")
            print(f"剩余余额: {result.balance}")
        except Exception as e:
            print(f"错误: {e}")

        # 示例 3：查询订单
        print("\n=== 查询订单 ===")
        try:
            order = client.query_order(
                openid="user_openid",
                session_key="user_session_key",
                order_id="ORDER_20240101_001",
            )
            print(f"订单 ID: {order.order_id}")
            print(f"状态: {order.status}")
            print(f"订单金额: {order.order_fee}")
        except Exception as e:
            print(f"错误: {e}")

    # 方式 2：手动管理生命周期
    print("\n=== 手动管理生命周期 ===")
    client = XPayClient(
        app_id="wx1234567890",
        app_key="your_app_key",
        env=0,
    )
    try:
        # 可以为不同用户使用不同的 session_key
        balance = client.query_user_balance(
            openid="user_openid",
            session_key="current_session_key",
        )
        print(f"余额: {balance.balance}")
    finally:
        client.close()


async def async_examples():
    """异步客户端使用示例。"""
    print("\n" + "=" * 60)
    print("异步客户端示例")
    print("=" * 60)

    # 方式 1：异步上下文管理器（推荐）
    async with XPayAsyncClient(
        app_id="wx1234567890",
        app_key="your_app_key",
        env=0,
    ) as client:
        # 示例 1：查询用户余额
        print("\n=== 异步查询用户余额 ===")
        try:
            balance = await client.query_user_balance(
                openid="user_openid",
                session_key="user_session_key",
            )
            print(f"余额: {balance.balance}")
            print(f"赠送余额: {balance.present_balance}")
        except Exception as e:
            print(f"错误: {e}")

        # 示例 2：并发处理多个支付（每个使用不同的 session_key）
        print("\n=== 并发处理多个支付 ===")
        tasks = [
            client.currency_pay(
                openid=f"user_{i}",
                session_key=f"session_key_{i}",  # 每个用户有自己的 session_key
                order_id=f"ORDER_20240101_00{i}",
                amount=100,
                payitem=f"道具_{i}",
            )
            for i in range(3)
        ]
        try:
            results = await asyncio.gather(*tasks, return_exceptions=True)
            for i, result in enumerate(results):
                if isinstance(result, Exception):
                    print(f"订单 {i} 错误: {result}")
                else:
                    print(f"订单 {i} ID: {result.order_id}")
        except Exception as e:
            print(f"错误: {e}")

        # 示例 3：查询订单
        print("\n=== 异步查询订单 ===")
        try:
            order = await client.query_order(
                openid="user_openid",
                session_key="user_session_key",
                order_id="ORDER_20240101_001",
            )
            print(f"订单 ID: {order.order_id}")
            print(f"状态: {order.status}")
        except Exception as e:
            print(f"错误: {e}")

    # 方式 2：手动管理生命周期
    print("\n=== 异步手动管理生命周期 ===")
    client = XPayAsyncClient(
        app_id="wx1234567890",
        app_key="your_app_key",
        env=0,
    )
    try:
        balance = await client.query_user_balance(
            openid="user_openid",
            session_key="current_session_key",
        )
        print(f"余额: {balance.balance}")
    finally:
        await client.close()


def webhook_example():
    """Webhook 处理示例。"""
    print("\n" + "=" * 60)
    print("Webhook 处理示例")
    print("=" * 60)

    parser = WebhookParser()

    # 示例 webhook 负载（来自微信）
    webhook_payload = {
        "ToUserName": "gh_1234567890",
        "FromUserName": "official_account_openid",
        "CreateTime": 1700000000,
        "MsgType": "event",
        "Event": "xpay_goods_deliver_notify",
        "OpenId": "user_openid_here",
        "OutTradeNo": "ORDER_20240101_001",
        "Env": 0,
        "WeChatPayInfo": {
            "MchOrderNo": "ORDER_20240101_001",
            "TransactionId": "4200001234567890",
            "PaidTime": 1700000000,
        },
        "GoodsInfo": {
            "ProductId": "product_001",
            "Quantity": 1,
            "OrigPrice": 100,
            "ActualPrice": 100,
            "Attach": "custom_data",
        },
    }

    print("\n=== 解析 Webhook ===")
    try:
        notification = parser.parse(webhook_payload)
        print(f"事件类型: {notification.event}")
        print(f"用户 OpenID: {notification.open_id}")
        print(f"订单号: {notification.out_trade_no}")

        if notification.goods_info:
            print(f"商品 ID: {notification.goods_info.product_id}")
            print(f"数量: {notification.goods_info.quantity}")

        # 返回成功响应
        success_response = parser.success_response()
        print(f"\n成功响应: {success_response}")
    except Exception as e:
        print(f"解析 Webhook 错误: {e}")


def main():
    """主函数。"""
    # 运行同步示例
    sync_examples()

    # 运行异步示例
    asyncio.run(async_examples())

    # 运行 Webhook 示例
    webhook_example()


if __name__ == "__main__":
    main()



"""XPay 客户端基础模块，包含共享逻辑。"""

from __future__ import annotations

import json
import logging
from typing import Any

from wechat_xpay.auth import calc_pay_sig, calc_signature
from wechat_xpay.exceptions import XPayAPIError


class BaseClient:
    """XPay 客户端基础类。

    包含配置和共享逻辑：签名计算、请求构建、响应解析、错误处理。

    Args:
        app_id: 小程序 AppID
        app_key: 用于计算 pay_sig 的 AppKey
        env: 环境，0 表示沙箱，1 表示生产环境
        base_url: 可选的自定义基础 URL
        logger: 可选的日志记录器，用于记录 API 调用和响应信息

    Note:
        access_token 和 session_key 不在初始化时传入，而是在每次调用 API 时传入，
        因为它们都有生命周期，会定期过期需要更新。
    """

    SANDBOX_BASE_URL = "https://api.weixin.qq.com"
    PROD_BASE_URL = "https://api.weixin.qq.com"

    def __init__(
        self,
        app_id: str,
        app_key: str,
        env: int = 1,
        base_url: str | None = None,
        logger: logging.Logger | None = None,
    ) -> None:
        self.app_id = app_id
        self.app_key = app_key
        self.env = env
        self.base_url = base_url or (self.PROD_BASE_URL if env == 1 else self.SANDBOX_BASE_URL)
        self.logger = logger

    def _prepare_request(
        self,
        endpoint: str,
        payload: dict[str, Any],
        access_token: str,
        session_key: str,
        *,
        needs_user_sig: bool = False,
    ) -> tuple[str, str, dict[str, str]]:
        """准备请求数据。

        添加公共字段、序列化请求体、计算签名、构建完整 URL。

        Args:
            endpoint: API 端点路径（如 '/xpay/query_user_balance'）
            payload: 请求体数据
            access_token: 接口调用凭证
            session_key: 用户的 session_key，用于计算用户态签名
            needs_user_sig: 是否需要用户态签名（signature），
                仅 query_user_balance 和 currency_pay 需要

        Returns:
            (完整 URL, 请求体字典, 请求头字典)
        """
        # 添加公共字段
        payload["env"] = self.env

        # 序列化请求体用于签名
        body_str = json.dumps(payload)

        # 签名消息 = uri + '&' + post_body
        sign_msg = endpoint + "&" + body_str

        # 计算支付签名（所有接口都需要）
        pay_sig = calc_pay_sig(endpoint, body_str, self.app_key)

        # 构建 URL
        url = f"{self.base_url}{endpoint}" f"?access_token={access_token}" f"&pay_sig={pay_sig}"

        # 仅在需要时追加用户态签名
        signature = None
        if needs_user_sig:
            signature = calc_signature(body_str, session_key)
            url += f"&signature={signature}"

        # 请求头
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
        }

        # 记录请求日志
        if self.logger:
            self.logger.debug(
                "XPay API Request: %s\n"
                "  URL: %s\n"
                "  POST Body: %s\n"
                "  Sign Message (uri&body): %s\n"
                "  pay_sig: %s\n"
                "  signature: %s\n"
                "  app_key: %s...%s\n"
                "  session_key: %s",
                endpoint,
                url,
                body_str,
                sign_msg,
                pay_sig,
                signature or "(not required)",
                self.app_key[:4],
                self.app_key[-4:],
                session_key[:4] + "..." + session_key[-4:] if len(session_key) > 8 else "****",
            )

        return url, body_str, headers

    def _handle_response(self, data: dict[str, Any]) -> dict[str, Any]:
        """处理响应数据。

        检查 API 错误码，返回业务数据。

        Args:
            data: 解析后的 JSON 响应

        Returns:
            业务数据（去除 errcode 和 errmsg）

        Raises:
            XPayAPIError: API 返回非零错误码
        """
        errcode = data.get("errcode", 0)
        errmsg = data.get("errmsg", "")

        # 记录响应日志
        if self.logger:
            if errcode != 0:
                self.logger.error(
                    "XPay API Error: errcode=%s, errmsg=%s\n" "  Response Body: %s",
                    errcode,
                    errmsg,
                    json.dumps(data, ensure_ascii=False),
                )
            else:
                self.logger.debug(
                    "XPay API Response: Success\n" "  Response Body: %s",
                    json.dumps(data, ensure_ascii=False),
                )

        if errcode != 0:
            raise XPayAPIError(errcode, errmsg)
        return {k: v for k, v in data.items() if k not in ("errcode", "errmsg")}



"""Webhook message parser for WeChat XPay push notifications."""

from __future__ import annotations

import xml.etree.ElementTree as ET
from dataclasses import dataclass
from typing import Any


@dataclass
class WechatPayInfo:
    """WeChat payment information in notifications."""

    mch_order_no: str = ""
    transaction_id: str = ""
    paid_time: int = 0


@dataclass
class GoodsInfo:
    """Goods information in delivery notification."""

    product_id: str = ""
    quantity: int = 0
    orig_price: int = 0
    actual_price: int = 0
    attach: str = ""


@dataclass
class CoinInfo:
    """Coin/token information in payment notification."""

    quantity: int = 0
    orig_price: int = 0
    actual_price: int = 0
    attach: str = ""


@dataclass
class TeamInfo:
    """Group buying information in notifications."""

    activity_id: str = ""
    team_id: str = ""
    team_type: int = 0
    team_action: int = 0


@dataclass
class GoodsDeliverNotify:
    """xpay_goods_deliver_notify message."""

    to_user_name: str = ""
    from_user_name: str = ""
    create_time: int = 0
    msg_type: str = ""
    event: str = ""
    open_id: str = ""
    out_trade_no: str = ""
    env: int = 0
    wechat_pay_info: WechatPayInfo | None = None
    goods_info: GoodsInfo | None = None
    team_info: TeamInfo | None = None


@dataclass
class CoinPayNotify:
    """xpay_coin_pay_notify message."""

    to_user_name: str = ""
    from_user_name: str = ""
    create_time: int = 0
    msg_type: str = ""
    event: str = ""
    open_id: str = ""
    out_trade_no: str = ""
    env: int = 0
    wechat_pay_info: WechatPayInfo | None = None
    coin_info: CoinInfo | None = None


@dataclass
class RefundNotify:
    """xpay_refund_notify message."""

    to_user_name: str = ""
    from_user_name: str = ""
    create_time: int = 0
    msg_type: str = ""
    event: str = ""
    open_id: str = ""
    wx_refund_id: str = ""
    mch_refund_id: str = ""
    wx_order_id: str = ""
    mch_order_id: str = ""
    refund_fee: int = 0
    ret_code: int = 0
    ret_msg: str = ""
    refund_start_timestamp: int = 0
    refund_succ_timestamp: int = 0
    wxpay_refund_transaction_id: str = ""
    retry_times: int = 0
    team_info: TeamInfo | None = None


@dataclass
class ComplaintNotify:
    """xpay_complaint_notify message."""

    to_user_name: str = ""
    from_user_name: str = ""
    create_time: int = 0
    msg_type: str = ""
    event: str = ""
    open_id: str = ""
    wx_order_id: str = ""
    mch_order_id: str = ""
    complaint_time: int = 0
    retry_times: int = 0
    request_id: str = ""


class WebhookParser:
    """Parser for WeChat XPay webhook messages."""

    def parse(
        self, payload: dict[str, Any] | str
    ) -> GoodsDeliverNotify | CoinPayNotify | RefundNotify | ComplaintNotify:
        """Parse webhook message.

        Args:
            payload: JSON dict or XML string from WeChat

        Returns:
            Parsed notification dataclass

        Raises:
            ValueError: If payload format is invalid
        """
        data = self._parse_xml(payload) if isinstance(payload, str) else payload

        event = data.get("Event")
        if event == "xpay_goods_deliver_notify":
            return self._parse_goods_deliver_notify(data)
        elif event == "xpay_coin_pay_notify":
            return self._parse_coin_pay_notify(data)
        elif event == "xpay_refund_notify":
            return self._parse_refund_notify(data)
        elif event == "xpay_complaint_notify":
            return self._parse_complaint_notify(data)
        else:
            raise ValueError(f"Unknown event type: {event}")

    def _parse_xml(self, xml_str: str) -> dict[str, Any]:
        """Parse XML string to dict."""
        root = ET.fromstring(xml_str)
        return self._xml_to_dict(root)

    def _xml_to_dict(self, element: ET.Element) -> dict[str, Any]:
        """Convert XML element to dictionary."""
        result: dict[str, Any] = {}
        for child in element:
            if len(child) > 0:
                result[child.tag] = self._xml_to_dict(child)
            else:
                result[child.tag] = child.text or ""
        return result

    def _parse_goods_deliver_notify(self, data: dict) -> GoodsDeliverNotify:
        """Parse goods delivery notification."""
        notify = GoodsDeliverNotify(
            to_user_name=data.get("ToUserName", ""),
            from_user_name=data.get("FromUserName", ""),
            create_time=int(data.get("CreateTime", 0)),
            msg_type=data.get("MsgType", ""),
            event=data.get("Event", ""),
            open_id=data.get("OpenId", ""),
            out_trade_no=data.get("OutTradeNo", ""),
            env=int(data.get("Env", 0)),
        )

        if "WeChatPayInfo" in data:
            wp = data["WeChatPayInfo"]
            notify.wechat_pay_info = WechatPayInfo(
                mch_order_no=wp.get("MchOrderNo", ""),
                transaction_id=wp.get("TransactionId", ""),
                paid_time=int(wp.get("PaidTime", 0)),
            )

        if "GoodsInfo" in data:
            gi = data["GoodsInfo"]
            notify.goods_info = GoodsInfo(
                product_id=gi.get("ProductId", ""),
                quantity=int(gi.get("Quantity", 0)),
                orig_price=int(gi.get("OrigPrice", 0)),
                actual_price=int(gi.get("ActualPrice", 0)),
                attach=gi.get("Attach", ""),
            )

        if "TeamInfo" in data:
            ti = data["TeamInfo"]
            notify.team_info = TeamInfo(
                activity_id=ti.get("ActivityId", ""),
                team_id=ti.get("TeamId", ""),
                team_type=int(ti.get("TeamType", 0)),
                team_action=int(ti.get("TeamAction", 0)),
            )

        return notify

    def _parse_coin_pay_notify(self, data: dict) -> CoinPayNotify:
        """Parse coin payment notification."""
        notify = CoinPayNotify(
            to_user_name=data.get("ToUserName", ""),
            from_user_name=data.get("FromUserName", ""),
            create_time=int(data.get("CreateTime", 0)),
            msg_type=data.get("MsgType", ""),
            event=data.get("Event", ""),
            open_id=data.get("OpenId", ""),
            out_trade_no=data.get("OutTradeNo", ""),
            env=int(data.get("Env", 0)),
        )

        if "WeChatPayInfo" in data:
            wp = data["WeChatPayInfo"]
            notify.wechat_pay_info = WechatPayInfo(
                mch_order_no=wp.get("MchOrderNo", ""),
                transaction_id=wp.get("TransactionId", ""),
                paid_time=int(wp.get("PaidTime", 0)),
            )

        if "CoinInfo" in data:
            ci = data["CoinInfo"]
            notify.coin_info = CoinInfo(
                quantity=int(ci.get("Quantity", 0)),
                orig_price=int(ci.get("OrigPrice", 0)),
                actual_price=int(ci.get("ActualPrice", 0)),
                attach=ci.get("Attach", ""),
            )

        return notify

    def _parse_refund_notify(self, data: dict) -> RefundNotify:
        """Parse refund notification."""
        notify = RefundNotify(
            to_user_name=data.get("ToUserName", ""),
            from_user_name=data.get("FromUserName", ""),
            create_time=int(data.get("CreateTime", 0)),
            msg_type=data.get("MsgType", ""),
            event=data.get("Event", ""),
            open_id=data.get("OpenId", ""),
            wx_refund_id=data.get("WxRefundId", ""),
            mch_refund_id=data.get("MchRefundId", ""),
            wx_order_id=data.get("WxOrderId", ""),
            mch_order_id=data.get("MchOrderId", ""),
            refund_fee=int(data.get("RefundFee", 0)),
            ret_code=int(data.get("RetCode", 0)),
            ret_msg=data.get("RetMsg", ""),
            refund_start_timestamp=int(data.get("RefundStartTimestamp", 0)),
            refund_succ_timestamp=int(data.get("RefundSuccTimestamp", 0)),
            wxpay_refund_transaction_id=data.get("WxpayRefundTransactionId", ""),
            retry_times=int(data.get("RetryTimes", 0)),
        )

        if "TeamInfo" in data:
            ti = data["TeamInfo"]
            notify.team_info = TeamInfo(
                activity_id=ti.get("ActivityId", ""),
                team_id=ti.get("TeamId", ""),
                team_type=int(ti.get("TeamType", 0)),
                team_action=int(ti.get("TeamAction", 0)),
            )

        return notify

    def _parse_complaint_notify(self, data: dict) -> ComplaintNotify:
        """Parse complaint notification."""
        return ComplaintNotify(
            to_user_name=data.get("ToUserName", ""),
            from_user_name=data.get("FromUserName", ""),
            create_time=int(data.get("CreateTime", 0)),
            msg_type=data.get("MsgType", ""),
            event=data.get("Event", ""),
            open_id=data.get("OpenId", ""),
            wx_order_id=data.get("WxOrderId", ""),
            mch_order_id=data.get("MchOrderId", ""),
            complaint_time=int(data.get("ComplaintTime", 0)),
            retry_times=int(data.get("RetryTimes", 0)),
            request_id=data.get("RequestId", ""),
        )



"""日志功能使用示例。"""

import logging

from config import ACCESS_TOKEN, APP_SECRET, APPID, ENV, OPENID, SESSION_KEY, USER_IP
from wechat_xpay import XPayAsyncClient, XPayClient


def setup_logger() -> logging.Logger:
    """配置日志记录器。"""
    logger = logging.getLogger("wechat_xpay")
    logger.setLevel(logging.DEBUG)

    # 控制台处理器
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG)

    # 文件处理器
    file_handler = logging.FileHandler("xpay.log", encoding="utf-8")
    file_handler.setLevel(logging.DEBUG)

    # 格式化器
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    console_handler.setFormatter(formatter)
    file_handler.setFormatter(formatter)

    logger.addHandler(console_handler)
    logger.addHandler(file_handler)

    return logger


def sync_example_with_logging():
    """同步客户端使用日志示例。"""
    logger = setup_logger()

    with XPayClient(
        app_id=APPID,
        app_key=APP_SECRET,
        env=ENV,  # 沙箱环境
        logger=logger,
    ) as client:
        try:
            # 查询用户余额
            balance = client.query_user_balance(
                openid=OPENID, access_token=ACCESS_TOKEN, session_key=SESSION_KEY, user_ip=USER_IP
            )
            print(f"用户余额: {balance.balance}")

            # 查询订单
            order = client.query_order(
                openid=OPENID,
                access_token=ACCESS_TOKEN,
                session_key=SESSION_KEY,
                order_id="order_123",
            )
            print(f"订单状态: {order.status}")

        except Exception as e:
            logger.exception("API 调用失败")
            print(f"错误: {e}")


async def async_example_with_logging():
    """异步客户端使用日志示例。"""
    logger = setup_logger()

    async with XPayAsyncClient(
        app_id=APPID,
        app_key=APP_SECRET,
        env=ENV,  # 沙箱环境
        logger=logger,
    ) as client:
        try:
            # 查询用户余额
            balance = await client.query_user_balance(
                openid=OPENID,
                access_token=ACCESS_TOKEN,
                session_key=SESSION_KEY,
            )
            print(f"用户余额: {balance.balance}")

            # 查询订单
            order = await client.query_order(
                openid=OPENID,
                access_token=ACCESS_TOKEN,
                session_key=SESSION_KEY,
                order_id="order_123",
            )
            print(f"订单状态: {order.status}")

        except Exception as e:
            logger.exception("API 调用失败")
            print(f"错误: {e}")


def example_without_logging():
    """不使用日志的示例（默认行为）。"""
    with XPayClient(
        app_id=APPID,
        app_key=APP_SECRET,
        env=ENV,
    ) as client:
        # 不传入 logger 参数，不会记录任何日志
        balance = client.query_user_balance(
            openid=OPENID,
            access_token=ACCESS_TOKEN,
            session_key=SESSION_KEY,
        )
        print(f"用户余额: {balance.balance}")


if __name__ == "__main__":
    print("=== 同步客户端日志示例 ===")
    sync_example_with_logging()

    print("\n=== 异步客户端日志示例 ===")
    import asyncio

    asyncio.run(async_example_with_logging())

    print("\n=== 不使用日志示例 ===")
    example_without_logging()



"""Custom exceptions for the WeChat XPay SDK."""

# Error code constants
ERR_SYSTEM_ERROR = -1
ERR_INVALID_OPENID = 268490001
ERR_BAD_REQUEST_PARAMS = 268490002
ERR_SIGNATURE_ERROR = 268490003
ERR_DUPLICATE_OPERATION = 268490004
ERR_ORDER_ALREADY_REFUNDED = 268490005
ERR_INSUFFICIENT_TOKEN_BALANCE = 268490006
ERR_SENSITIVE_CONTENT = 268490007
ERR_TOKEN_NOT_PUBLISHED = 268490008
ERR_SESSION_KEY_EXPIRED = 268490009
ERR_DATA_GENERATING = 268490011
ERR_BATCH_TASK_RUNNING = 268490012
ERR_CANNOT_REFUND_VERIFIED_ORDER = 268490013
ERR_REFUND_IN_PROGRESS = 268490014
ERR_RATE_LIMIT_EXCEEDED = 268490015
ERR_LEFT_FEE_MISMATCH = 268490016
ERR_INVALID_ENV = 268447747
ERR_INVALID_APPID = 268447748
ERR_MCH_NO_PERMISSION = 268447749
ERR_USER_NOT_REGISTERED = 268500000
ERR_INVALID_SIGN_TYPE = 268500001
ERR_INVALID_BODY_PARAM = 268500002
ERR_APPID_MCHID_MISMATCH = 268500003
ERR_RESOURCE_NOT_EXIST = 268500004
ERR_SIGN_VERIFICATION_FAILED = 268500005
ERR_MCH_NOT_EXISTS = 268500006
ERR_MCH_ACCOUNT_ABNORMAL = 268500007
ERR_AMOUNT_MISMATCH = 268500008
ERR_SERVICE_NOT_ENABLED = 268500009
ERR_REFUND_FEE_EXCEEDED = 268500010
ERR_ORDER_PAID = 268500011
ERR_ORDER_CLOSED = 268500012
ERR_MCH_CONFIG_LIMITED = 268500013
ERR_REAL_NAME_REQUIRED = 268500014
ERR_BANK_CARD_NOT_SUPPORTED = 268500015
ERR_ACCOUNT_FROZEN = 268500016
ERR_ORDER_NOT_EXIST = 268500017
ERR_REFUND_NOT_EXIST = 268500018
ERR_REFUND_AMOUNT_LIMIT = 268500019
ERR_INVALID_TRANSACTION_ID = 268500020
ERR_BALANCE_NOT_ENOUGH = 268500021
ERR_REFUND_FEE_LIMIT = 268500022
ERR_SYSTEM_BUSY = 268500023
ERR_UPLOAD_FILE_FAILED = 268500024
ERR_INVALID_FILE_TYPE = 268500025
ERR_FILE_SIZE_EXCEEDED = 268500026
ERR_INVALID_GOODS_CONFIG = 268500027
ERR_GOODS_NOT_PUBLISHED = 268500028
ERR_BATCH_TASK_FAILED = 268500029
ERR_WITHDRAW_AMOUNT_LIMIT = 268500030
ERR_WITHDRAW_FEE_LIMIT = 268500031
ERR_WITHDRAW_COUNT_LIMIT = 268500032
ERR_TRANSFER_ACCOUNT_NOT_BOUND = 268500033
ERR_ADVER_FUNDS_NOT_ENOUGH = 268500034
ERR_BILL_GENERATING = 268500035
ERR_REFUND_APPLICATION_EXIST = 268500036
ERR_COMPLAINT_NOT_EXIST = 268500037
ERR_NEGOTIATION_FAILED = 268500038
ERR_INVALID_MEDIA_TYPE = 268500039
ERR_MEDIA_UPLOAD_FAILED = 268500040
ERR_IMAGE_SIZE_EXCEEDED = 268500041
ERR_IMAGE_FORMAT_NOT_SUPPORTED = 268500042
ERR_VIDEO_SIZE_EXCEEDED = 268500043
ERR_VIDEO_FORMAT_NOT_SUPPORTED = 268500044


class XPayError(Exception):
    """Base exception for all SDK errors."""


class XPayAPIError(XPayError):
    """Raised when the WeChat XPay API returns a non-zero errcode.

    Attributes:
        errcode: WeChat error code integer.
        errmsg:  Human-readable error message from the API.

    Common error codes:
        -1 (ERR_SYSTEM_ERROR)
            System error
        268490001 (ERR_INVALID_OPENID)
            Invalid openid
        268490002 (ERR_BAD_REQUEST_PARAMS)
            Bad request parameters (see errmsg)
        268490003 (ERR_SIGNATURE_ERROR)
            Signature error
        268490004 (ERR_DUPLICATE_OPERATION)
            Duplicate operation — treat as success for present_currency
        268490005 (ERR_ORDER_ALREADY_REFUNDED)
            Order already refunded via cancel_currency_pay
        268490006 (ERR_INSUFFICIENT_TOKEN_BALANCE)
            Insufficient token balance for refund/payment
        268490007 (ERR_SENSITIVE_CONTENT)
            Sensitive content in image or text
        268490008 (ERR_TOKEN_NOT_PUBLISHED)
            Token not published yet
        268490009 (ERR_SESSION_KEY_EXPIRED)
            session_key missing or expired — caller must re-login
        268490011 (ERR_DATA_GENERATING)
            Data still generating — poll again
        268490012 (ERR_BATCH_TASK_RUNNING)
            Batch task running — wait for completion
        268490013 (ERR_CANNOT_REFUND_VERIFIED_ORDER)
            Cannot refund a verified order
        268490014 (ERR_REFUND_IN_PROGRESS)
            Refund in progress — retry with same params
        268490015 (ERR_RATE_LIMIT_EXCEEDED)
            Rate limit exceeded
        268490016 (ERR_LEFT_FEE_MISMATCH)
            left_fee mismatch — query order to confirm
    """

    def __init__(self, errcode: int, errmsg: str) -> None:
        self.errcode = errcode
        self.errmsg = errmsg
        super().__init__(f"XPay API error {errcode}: {errmsg}")



"""Tests for authentication module."""

from wechat_xpay.auth import calc_pay_sig, calc_signature


class TestAuth:
    """Test suite for authentication functions."""

    def test_calc_pay_sig(self):
        """Test pay_sig calculation."""
        uri = "/xpay/query_user_balance"
        post_body = '{"openid":"test_openid","appid":"wx123","env":0}'
        app_key = "test_app_key"

        result = calc_pay_sig(uri, post_body, app_key)

        # Verify it's a valid hex string
        assert len(result) == 64  # SHA-256 produces 32 bytes = 64 hex chars
        int(result, 16)  # Should not raise

    def test_calc_signature(self):
        """Test signature calculation."""
        post_body = '{"openid":"test_openid","appid":"wx123","env":0}'
        session_key = "test_session_key"

        result = calc_signature(post_body, session_key)

        # Verify it's a valid hex string
        assert len(result) == 64
        int(result, 16)

    def test_pay_sig_consistency(self):
        """Test that pay_sig is consistent for same input."""
        uri = "/xpay/query_user_balance"
        post_body = '{"openid":"test_openid"}'
        app_key = "test_key"

        sig1 = calc_pay_sig(uri, post_body, app_key)
        sig2 = calc_pay_sig(uri, post_body, app_key)

        assert sig1 == sig2

    def test_signature_consistency(self):
        """Test that signature is consistent for same input."""
        post_body = '{"openid":"test_openid"}'
        session_key = "test_key"

        sig1 = calc_signature(post_body, session_key)
        sig2 = calc_signature(post_body, session_key)

        assert sig1 == sig2



"""Dataclasses for WeChat XPay API response objects."""

from __future__ import annotations

from dataclasses import dataclass, field

# ---------------------------------------------------------------------------
# query_user_balance
# ---------------------------------------------------------------------------


@dataclass
class UserBalance:
    balance: int
    present_balance: int
    sum_save: int
    sum_present: int
    sum_balance: int
    sum_cost: int
    first_save_flag: bool


# ---------------------------------------------------------------------------
# currency_pay
# ---------------------------------------------------------------------------


@dataclass
class CurrencyPayResult:
    order_id: str
    balance: int
    used_present_amount: int


# ---------------------------------------------------------------------------
# query_order — nested Order object
# ---------------------------------------------------------------------------


@dataclass
class Order:
    order_id: str
    create_time: int
    update_time: int
    status: int
    biz_type: int
    order_fee: int
    paid_fee: int
    order_type: int
    env_type: int
    coupon_fee: int | None = None
    refund_fee: int | None = None
    paid_time: int | None = None
    provide_time: int | None = None
    biz_meta: str | None = None
    token: str | None = None
    left_fee: int | None = None
    wx_order_id: str | None = None
    channel_order_id: str | None = None
    wxpay_order_id: str | None = None
    sett_time: int | None = None
    sett_state: int | None = None
    platform_fee_fen: int | None = None
    cps_fee_fen: int | None = None


# ---------------------------------------------------------------------------
# cancel_currency_pay
# ---------------------------------------------------------------------------


@dataclass
class CancelCurrencyPayResult:
    order_id: str


# ---------------------------------------------------------------------------
# present_currency
# ---------------------------------------------------------------------------


@dataclass
class PresentCurrencyResult:
    balance: int
    order_id: str
    present_balance: int


# ---------------------------------------------------------------------------
# download_bill / download_adverfunds_order
# ---------------------------------------------------------------------------


@dataclass
class BillDownload:
    url: str


# ---------------------------------------------------------------------------
# refund_order
# ---------------------------------------------------------------------------


@dataclass
class RefundOrderResult:
    refund_order_id: str
    refund_wx_order_id: str
    pay_order_id: str
    pay_wx_order_id: str


# ---------------------------------------------------------------------------
# create_withdraw_order
# ---------------------------------------------------------------------------


@dataclass
class WithdrawOrderResult:
    withdraw_no: str
    wx_withdraw_no: str


# ---------------------------------------------------------------------------
# query_withdraw_order
# ---------------------------------------------------------------------------


@dataclass
class WithdrawOrder:
    withdraw_no: str
    status: int
    withdraw_amount: str
    wx_withdraw_no: str
    withdraw_success_timestamp: str | None = None
    create_time: str | None = None
    fail_reason: str | None = None


# ---------------------------------------------------------------------------
# start_upload_goods / query_upload_goods
# ---------------------------------------------------------------------------


@dataclass
class GoodsUploadItem:
    id: str
    name: str
    price: int
    remark: str
    item_url: str
    upload_status: int | None = None
    errmsg: str | None = None


@dataclass
class GoodsUploadStatus:
    status: int
    upload_item: list[GoodsUploadItem] = field(default_factory=list)


# ---------------------------------------------------------------------------
# start_publish_goods / query_publish_goods
# ---------------------------------------------------------------------------


@dataclass
class GoodsPublishItem:
    id: str
    publish_status: int | None = None
    errmsg: str | None = None


@dataclass
class GoodsPublishStatus:
    status: int
    publish_item: list[GoodsPublishItem] = field(default_factory=list)


# ---------------------------------------------------------------------------
# query_biz_balance
# ---------------------------------------------------------------------------


@dataclass
class BizBalanceAvailable:
    amount: str
    currency_code: str


@dataclass
class BizBalance:
    balance_available: BizBalanceAvailable


# ---------------------------------------------------------------------------
# query_transfer_account
# ---------------------------------------------------------------------------


@dataclass
class TransferAccount:
    transfer_account_name: str
    transfer_account_uid: int
    transfer_account_agency_id: int
    transfer_account_agency_name: str
    state: int
    bind_result: int
    error_msg: str | None = None


# ---------------------------------------------------------------------------
# query_adver_funds
# ---------------------------------------------------------------------------


@dataclass
class AdverFund:
    settle_begin: int
    settle_end: int
    total_amount: int
    remain_amount: int
    expire_time: int
    fund_type: int
    fund_id: str


@dataclass
class AdverFundList:
    adver_funds_list: list[AdverFund]
    total_page: int


# ---------------------------------------------------------------------------
# create_funds_bill
# ---------------------------------------------------------------------------


@dataclass
class FundsBillResult:
    bill_id: str


# ---------------------------------------------------------------------------
# query_funds_bill
# ---------------------------------------------------------------------------


@dataclass
class FundsBillItem:
    bill_id: str
    oper_time: int
    settle_begin: int
    settle_end: int
    fund_id: str
    transfer_account_name: str
    transfer_account_uid: int
    transfer_amount: int
    status: int
    request_id: str


@dataclass
class FundsBillList:
    bill_list: list[FundsBillItem]
    total_page: int


# ---------------------------------------------------------------------------
# query_recover_bill
# ---------------------------------------------------------------------------


@dataclass
class RecoverBillItem:
    bill_id: str
    recover_time: int
    settle_begin: int
    settle_end: int
    fund_id: str
    recover_account_name: str
    recover_amount: int
    refund_order_list: list[str] = field(default_factory=list)


@dataclass
class RecoverBillList:
    bill_list: list[RecoverBillItem]
    total_page: int


# ---------------------------------------------------------------------------
# get_complaint_list / get_complaint_detail
# ---------------------------------------------------------------------------


@dataclass
class ComplaintOrderInfo:
    transaction_id: str
    out_trade_no: str
    amount: int
    wxa_out_trade_no: str
    wx_order_id: str


@dataclass
class ComplaintMedia:
    media_type: str
    media_url: list[str] = field(default_factory=list)


@dataclass
class ServiceOrderInfo:
    order_id: str
    out_order_no: str
    state: str


@dataclass
class Complaint:
    complaint_id: str
    complaint_time: str
    complaint_detail: str
    complaint_state: str
    payer_openid: str
    complaint_order_info: list[ComplaintOrderInfo] = field(default_factory=list)
    complaint_full_refunded: bool = False
    incoming_user_response: bool = False
    user_complaint_times: int = 0
    complaint_media_list: list[ComplaintMedia] = field(default_factory=list)
    problem_description: str | None = None
    problem_type: str | None = None
    apply_refund_amount: int | None = None
    user_tag_list: list[str] = field(default_factory=list)
    service_order_info: list[ServiceOrderInfo] = field(default_factory=list)
    payer_phone: str | None = None


@dataclass
class ComplaintList:
    total: int
    complaints: list[Complaint]


# ---------------------------------------------------------------------------
# get_negotiation_history
# ---------------------------------------------------------------------------


@dataclass
class NegotiationRecord:
    log_id: str
    operator: str
    operate_time: str
    operate_type: str
    operate_details: str
    complaint_media_list: list[ComplaintMedia] = field(default_factory=list)


@dataclass
class NegotiationHistory:
    total: int
    history: list[NegotiationRecord]


# ---------------------------------------------------------------------------
# upload_vp_file / get_upload_file_sign
# ---------------------------------------------------------------------------


@dataclass
class UploadFileResult:
    file_id: str


@dataclass
class UploadFileSign:
    sign: str
    cos_url: str | None = None


# ---------------------------------------------------------------------------
# notify_provide_goods
# ---------------------------------------------------------------------------


@dataclass
class NotifyProvideGoodsResult:
    order_id: str
    out_trade_no: str
    provide_status: int


# ---------------------------------------------------------------------------
# download_adverfunds_order
# ---------------------------------------------------------------------------


@dataclass
class AdverfundsOrderDownload:
    url: str



"""同步客户端测试。"""

import pytest
import respx
from httpx import Response

from wechat_xpay import XPayClient, models
from wechat_xpay.exceptions import XPayAPIError


class TestXPayClient:
    """测试 XPayClient 同步客户端。"""

    @pytest.fixture
    def client(self):
        """提供同步客户端 fixture。"""
        return XPayClient(
            app_id="wx1234567890",
            app_key="test_app_key",
            env=0,  # Sandbox
        )

    @respx.mock
    def test_query_user_balance(self, client):
        """测试查询用户余额。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance": 1000,
                    "present_balance": 500,
                    "sum_save": 2000,
                    "sum_present": 1000,
                    "sum_balance": 3000,
                    "sum_cost": 1000,
                    "first_save_flag": False,
                },
            )
        )

        result = client.query_user_balance(
            openid="test_openid",
            access_token="test_access_token",
            session_key="test_session_key",
        )

        assert result.balance == 1000
        assert result.present_balance == 500
        assert route.called

    @respx.mock
    def test_api_error(self, client):
        """测试 API 错误处理。"""
        respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 268490001,
                    "errmsg": "Invalid openid",
                },
            )
        )

        with pytest.raises(XPayAPIError) as exc_info:
            client.query_user_balance(
                openid="invalid_openid",
                access_token="test_access_token",
                session_key="test_session_key",
            )

        assert exc_info.value.errcode == 268490001
        assert "Invalid openid" in exc_info.value.errmsg

    def test_context_manager(self):
        """测试同步上下文管理器。"""
        with XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        ) as client:
            assert client.app_id == "wx123"
            assert client._client is not None


class TestOrderAPIs:
    """测试订单管理 API。"""

    @pytest.fixture
    def client(self):
        """提供同步客户端 fixture。"""
        return XPayClient(
            app_id="wx1234567890",
            app_key="test_app_key",
            env=0,
        )

    @respx.mock
    def test_query_order(self, client):
        """测试查询订单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "order_123",
                    "create_time": 1700000000,
                    "update_time": 1700000100,
                    "status": 2,
                    "biz_type": 0,
                    "order_fee": 1000,
                    "paid_fee": 1000,
                    "order_type": 0,
                    "env_type": 1,
                },
            )
        )

        result = client.query_order(
            openid="user_123",
            access_token="test_access_token",
            session_key="test_session_key",
            order_id="order_123",
        )

        assert isinstance(result, models.Order)
        assert result.order_id == "order_123"
        assert result.status == 2
        assert route.called

    @respx.mock
    def test_cancel_currency_pay(self, client):
        """测试取消代币支付。"""
        route = respx.post("https://api.weixin.qq.com/xpay/cancel_currency_pay").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "refund_order_123",
                },
            )
        )

        result = client.cancel_currency_pay(
            openid="user_123",
            access_token="test_access_token",
            session_key="test_session_key",
            pay_order_id="original_order_123",
            order_id="refund_order_123",
            amount=100,
        )

        assert isinstance(result, models.CancelCurrencyPayResult)
        assert result.order_id == "refund_order_123"
        assert route.called

    @respx.mock
    def test_present_currency(self, client):
        """测试赠送代币。"""
        route = respx.post("https://api.weixin.qq.com/xpay/present_currency").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance": 1200,
                    "order_id": "present_order_123",
                    "present_balance": 400,
                },
            )
        )

        result = client.present_currency(
            openid="user_123",
            access_token="test_access_token",
            session_key="test_session_key",
            order_id="present_order_123",
            amount=200,
        )

        assert isinstance(result, models.PresentCurrencyResult)
        assert result.balance == 1200
        assert result.present_balance == 400
        assert route.called


class TestRefundAndWithdrawAPIs:
    """测试退款和提现 API。"""

    @pytest.fixture
    def client(self):
        """提供同步客户端 fixture。"""
        return XPayClient(
            app_id="wx1234567890",
            app_key="test_app_key",
            env=0,
        )

    @respx.mock
    def test_refund_order(self, client):
        """测试退款订单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/refund_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "refund_order_id": "refund_123",
                    "refund_wx_order_id": "wx_refund_456",
                    "pay_order_id": "order_123",
                    "pay_wx_order_id": "wx_order_456",
                },
            )
        )

        result = client.refund_order(
            openid="user_123",
            access_token="test_access_token",
            session_key="test_session_key",
            refund_order_id="refund_123",
            left_fee=1000,
            refund_fee=500,
            refund_reason="1",
            req_from="1",
            order_id="original_order",
        )

        assert isinstance(result, models.RefundOrderResult)
        assert result.refund_order_id == "refund_123"
        assert route.called

    @respx.mock
    def test_create_withdraw_order(self, client):
        """测试创建提现订单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/create_withdraw_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "withdraw_no": "withdraw_123",
                    "wx_withdraw_no": "wx_withdraw_456",
                },
            )
        )

        result = client.create_withdraw_order(
            access_token="test_access_token",
            session_key="test_session_key",
            withdraw_no="withdraw_123",
            withdraw_amount="100.00",
        )

        assert isinstance(result, models.WithdrawOrderResult)
        assert result.withdraw_no == "withdraw_123"
        assert route.called

    @respx.mock
    def test_query_withdraw_order(self, client):
        """测试查询提现订单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_withdraw_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "withdraw_no": "withdraw_123",
                    "status": 2,
                    "withdraw_amount": "100.00",
                    "wx_withdraw_no": "wx_withdraw_456",
                },
            )
        )

        result = client.query_withdraw_order(
            access_token="test_access_token",
            session_key="test_session_key",
            withdraw_no="withdraw_123",
        )

        assert isinstance(result, models.WithdrawOrder)
        assert result.status == 2
        assert route.called

    @respx.mock
    def test_download_bill(self, client):
        """测试下载账单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/download_bill").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "url": "https://example.com/bill.csv",
                },
            )
        )

        result = client.download_bill(
            access_token="test_access_token",
            session_key="test_session_key",
            begin_ds=20230801,
            end_ds=20230810,
        )

        assert isinstance(result, models.BillDownload)
        assert result.url == "https://example.com/bill.csv"
        assert route.called


class TestAdvertisingFundAPIs:
    """测试广告金相关 API。"""

    @pytest.fixture
    def client(self):
        """提供同步客户端 fixture。"""
        return XPayClient(
            app_id="wx1234567890",
            app_key="test_app_key",
            env=0,
        )

    @respx.mock
    def test_query_biz_balance(self, client):
        """测试查询商家余额。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_biz_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance_available": {
                        "amount": "1000.00",
                        "currency_code": "CNY",
                    },
                },
            )
        )

        result = client.query_biz_balance(
            access_token="test_access_token", session_key="test_session_key"
        )

        assert isinstance(result, models.BizBalance)
        assert result.balance_available.amount == "1000.00"
        assert route.called

    @respx.mock
    def test_query_transfer_account(self, client):
        """测试查询转账账户。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_transfer_account").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "acct_list": [
                        {
                            "transfer_account_name": "Account 1",
                            "transfer_account_uid": 12345,
                            "transfer_account_agency_id": 67890,
                            "transfer_account_agency_name": "Agency 1",
                            "state": 1,
                            "bind_result": 1,
                        }
                    ],
                },
            )
        )

        result = client.query_transfer_account(
            access_token="test_access_token", session_key="test_session_key"
        )

        assert isinstance(result, list)
        assert len(result) == 1
        assert isinstance(result[0], models.TransferAccount)
        assert result[0].transfer_account_name == "Account 1"
        assert route.called

    @respx.mock
    def test_query_adver_funds(self, client):
        """测试查询广告金。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_adver_funds").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total_page": 1,
                    "adver_funds_list": [
                        {
                            "settle_begin": 1700000000,
                            "settle_end": 1700086400,
                            "total_amount": 10000,
                            "remain_amount": 5000,
                            "expire_time": 1702678400,
                            "fund_type": 0,
                            "fund_id": "fund_123",
                        }
                    ],
                },
            )
        )

        result = client.query_adver_funds(
            access_token="test_access_token",
            session_key="test_session_key",
            page=1,
            page_size=10,
        )

        assert isinstance(result, models.AdverFundList)
        assert result.total_page == 1
        assert len(result.adver_funds_list) == 1
        assert route.called


class TestComplaintAPIs:
    """测试投诉管理 API。"""

    @pytest.fixture
    def client(self):
        """提供同步客户端 fixture。"""
        return XPayClient(
            app_id="wx1234567890",
            app_key="test_app_key",
            env=0,
        )

    @respx.mock
    def test_get_complaint_list(self, client):
        """测试获取投诉列表。"""
        route = respx.post("https://api.weixin.qq.com/xpay/get_complaint_list").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total": 1,
                    "complaints": [
                        {
                            "complaint_id": "complaint_123",
                            "complaint_time": "2023-11-28T11:11:49+08:00",
                            "complaint_detail": "Test complaint",
                            "complaint_state": "PENDING",
                            "payer_openid": "openid_123",
                        }
                    ],
                },
            )
        )

        result = client.get_complaint_list(
            access_token="test_access_token",
            session_key="test_session_key",
            begin_date="2023-01-01",
            end_date="2023-12-31",
            offset=0,
            limit=10,
        )

        assert isinstance(result, models.ComplaintList)
        assert result.total == 1
        assert len(result.complaints) == 1
        assert route.called

    @respx.mock
    def test_get_complaint_detail(self, client):
        """测试获取投诉详情。"""
        route = respx.post("https://api.weixin.qq.com/xpay/get_complaint_detail").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "complaint": {
                        "complaint_id": "complaint_123",
                        "complaint_time": "2023-11-28T11:11:49+08:00",
                        "complaint_detail": "Test complaint",
                        "complaint_state": "PENDING",
                        "payer_openid": "openid_123",
                    },
                },
            )
        )

        result = client.get_complaint_detail(
            access_token="test_access_token",
            session_key="test_session_key",
            complaint_id="complaint_123",
        )

        assert isinstance(result, models.Complaint)
        assert result.complaint_id == "complaint_123"
        assert route.called

    @respx.mock
    def test_response_complaint(self, client):
        """测试回复投诉。"""
        route = respx.post("https://api.weixin.qq.com/xpay/response_complaint").mock(
            return_value=Response(
                200,
                json={"errcode": 0, "errmsg": "ok"},
            )
        )

        result = client.response_complaint(
            access_token="test_access_token",
            session_key="test_session_key",
            complaint_id="complaint_123",
            response_content="We will process this soon",
        )

        assert result is True
        assert route.called

    @respx.mock
    def test_complete_complaint(self, client):
        """测试完成投诉处理。"""
        route = respx.post("https://api.weixin.qq.com/xpay/complete_complaint").mock(
            return_value=Response(
                200,
                json={"errcode": 0, "errmsg": "ok"},
            )
        )

        result = client.complete_complaint(
            access_token="test_access_token",
            session_key="test_session_key",
            complaint_id="complaint_123",
        )

        assert result is True
        assert route.called

    @respx.mock
    def test_upload_vp_file(self, client):
        """测试上传文件。"""
        route = respx.post("https://api.weixin.qq.com/xpay/upload_vp_file").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "file_id": "file_123",
                },
            )
        )

        result = client.upload_vp_file(
            access_token="test_access_token",
            session_key="test_session_key",
            file_name="test.jpg",
            img_url="https://example.com/image.jpg",
        )

        assert isinstance(result, models.UploadFileResult)
        assert result.file_id == "file_123"
        assert route.called



"""测试新增的 API 实现。"""

import respx
from httpx import Response

from wechat_xpay import XPayAsyncClient, XPayClient, models


class TestNotifyProvideGoods:
    """测试发货完成通知 API。"""

    @respx.mock
    def test_notify_provide_goods_sync(self):
        """测试同步发货通知。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/notify_provide_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "order_123",
                    "out_trade_no": "OUT_001",
                    "provide_status": 1,
                },
            )
        )

        result = client.notify_provide_goods(
            access_token="test_access_token",
            session_key="session_key_456",
            order_id="order_123",
        )

        assert isinstance(result, models.NotifyProvideGoodsResult)
        assert result.order_id == "order_123"
        assert result.out_trade_no == "OUT_001"
        assert result.provide_status == 1
        assert route.called

    @respx.mock
    async def test_notify_provide_goods_async(self):
        """测试异步发货通知。"""
        client = XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/notify_provide_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "order_123",
                    "out_trade_no": "OUT_001",
                    "provide_status": 1,
                },
            )
        )

        result = await client.notify_provide_goods(
            access_token="test_access_token",
            session_key="session_key_456",
            wx_order_id="wx_order_123",
        )

        assert isinstance(result, models.NotifyProvideGoodsResult)
        assert result.provide_status == 1
        assert route.called
        await client.close()


class TestGoodsManagement:
    """测试道具管理 API。"""

    @respx.mock
    def test_start_upload_goods_sync(self):
        """测试同步启动道具上传。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/start_upload_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "status": 1,
                    "upload_item": [
                        {
                            "id": "goods_001",
                            "name": "道具1",
                            "price": 100,
                            "remark": "测试道具",
                            "item_url": "https://example.com/item1",
                            "upload_status": 0,
                        }
                    ],
                },
            )
        )

        result = client.start_upload_goods(
            access_token="test_access_token",
            session_key="session_key_456",
            goods=[
                {
                    "id": "goods_001",
                    "name": "道具1",
                    "price": 100,
                    "remark": "测试道具",
                    "item_url": "https://example.com/item1",
                }
            ],
        )

        assert isinstance(result, models.GoodsUploadStatus)
        assert result.status == 1
        assert len(result.upload_item) == 1
        assert result.upload_item[0].id == "goods_001"
        assert route.called

    @respx.mock
    def test_query_upload_goods_sync(self):
        """测试同步查询道具上传状态。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/query_upload_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "status": 2,
                    "upload_item": [
                        {
                            "id": "goods_001",
                            "name": "道具1",
                            "price": 100,
                            "remark": "测试道具",
                            "item_url": "https://example.com/item1",
                            "upload_status": 1,
                        }
                    ],
                },
            )
        )

        result = client.query_upload_goods(
            access_token="test_access_token", session_key="session_key_456"
        )

        assert isinstance(result, models.GoodsUploadStatus)
        assert result.status == 2
        assert result.upload_item[0].upload_status == 1
        assert route.called

    @respx.mock
    def test_start_publish_goods_sync(self):
        """测试同步启动道具发布。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/start_publish_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "status": 1,
                    "publish_item": [
                        {"id": "goods_001", "publish_status": 0},
                        {"id": "goods_002", "publish_status": 0},
                    ],
                },
            )
        )

        result = client.start_publish_goods(
            access_token="test_access_token",
            session_key="session_key_456",
            goods=[{"id": "goods_001"}, {"id": "goods_002"}],
        )

        assert isinstance(result, models.GoodsPublishStatus)
        assert result.status == 1
        assert len(result.publish_item) == 2
        assert route.called

    @respx.mock
    def test_query_publish_goods_sync(self):
        """测试同步查询道具发布状态。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/query_publish_goods").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "status": 2,
                    "publish_item": [
                        {"id": "goods_001", "publish_status": 1},
                    ],
                },
            )
        )

        result = client.query_publish_goods(
            access_token="test_access_token", session_key="session_key_456"
        )

        assert isinstance(result, models.GoodsPublishStatus)
        assert result.status == 2
        assert result.publish_item[0].publish_status == 1
        assert route.called


class TestFundsBillAPIs:
    """测试广告金账单 API。"""

    @respx.mock
    def test_create_funds_bill_sync(self):
        """测试同步创建广告金账单。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/create_funds_bill").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "bill_id": "bill_123",
                },
            )
        )

        result = client.create_funds_bill(
            access_token="test_access_token",
            session_key="session_key_456",
            transfer_account_uid=12345,
            transfer_account_agency_id=67890,
            transfer_amount=10000,
            fund_id="fund_001",
            settle_begin=1700000000,
            settle_end=1700086400,
        )

        assert isinstance(result, models.FundsBillResult)
        assert result.bill_id == "bill_123"
        assert route.called

    @respx.mock
    def test_bind_transfer_account_sync(self):
        """测试同步绑定转账账户。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/bind_transfer_accout").mock(
            return_value=Response(
                200,
                json={"errcode": 0, "errmsg": "ok"},
            )
        )

        result = client.bind_transfer_account(
            access_token="test_access_token",
            session_key="session_key_456",
            transfer_account_uid=12345,
            transfer_account_agency_id=67890,
        )

        assert result is True
        assert route.called

    @respx.mock
    def test_query_funds_bill_sync(self):
        """测试同步查询资金账单。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/query_funds_bill").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total_page": 1,
                    "bill_list": [
                        {
                            "bill_id": "bill_001",
                            "oper_time": 1700000000,
                            "settle_begin": 1700000000,
                            "settle_end": 1700086400,
                            "fund_id": "fund_001",
                            "transfer_account_name": "Account 1",
                            "transfer_account_uid": 12345,
                            "transfer_amount": 10000,
                            "status": 1,
                            "request_id": "req_001",
                        }
                    ],
                },
            )
        )

        result = client.query_funds_bill(
            access_token="test_access_token",
            session_key="session_key_456",
            page=1,
            page_size=10,
        )

        assert isinstance(result, models.FundsBillList)
        assert result.total_page == 1
        assert len(result.bill_list) == 1
        assert result.bill_list[0].bill_id == "bill_001"
        assert route.called

    @respx.mock
    def test_query_recover_bill_sync(self):
        """测试同步查询回收账单。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/query_recover_bill").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total_page": 1,
                    "bill_list": [
                        {
                            "bill_id": "recover_001",
                            "recover_time": 1700000000,
                            "settle_begin": 1700000000,
                            "settle_end": 1700086400,
                            "fund_id": "fund_001",
                            "recover_account_name": "Account 1",
                            "recover_amount": 5000,
                            "refund_order_list": ["order_001", "order_002"],
                        }
                    ],
                },
            )
        )

        result = client.query_recover_bill(
            access_token="test_access_token",
            session_key="session_key_456",
            page=1,
            page_size=10,
        )

        assert isinstance(result, models.RecoverBillList)
        assert result.total_page == 1
        assert len(result.bill_list) == 1
        assert result.bill_list[0].bill_id == "recover_001"
        assert len(result.bill_list[0].refund_order_list) == 2
        assert route.called

    @respx.mock
    def test_download_adverfunds_order_sync(self):
        """测试同步下载广告金订单账单。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/download_adverfunds_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "url": "https://example.com/adverfunds_order.csv",
                },
            )
        )

        result = client.download_adverfunds_order(
            access_token="test_access_token",
            session_key="session_key_456",
            begin_ds=20230801,
            end_ds=20230810,
        )

        assert isinstance(result, models.AdverfundsOrderDownload)
        assert result.url == "https://example.com/adverfunds_order.csv"
        assert route.called


class TestNegotiationHistory:
    """测试协商历史 API。"""

    @respx.mock
    def test_get_negotiation_history_sync(self):
        """测试同步获取协商历史。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/get_negotiation_history").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total": 2,
                    "history": [
                        {
                            "log_id": "log_001",
                            "operator": "user",
                            "operate_time": "2023-11-28T10:00:00+08:00",
                            "operate_type": "APPLY_REFUND",
                            "operate_details": "申请退款",
                        },
                        {
                            "log_id": "log_002",
                            "operator": "mch",
                            "operate_time": "2023-11-28T11:00:00+08:00",
                            "operate_type": "RESPONSE",
                            "operate_details": "商家回复",
                        },
                    ],
                },
            )
        )

        result = client.get_negotiation_history(
            access_token="test_access_token",
            session_key="session_key_456",
            complaint_id="complaint_123",
            offset=0,
            limit=10,
        )

        assert isinstance(result, models.NegotiationHistory)
        assert result.total == 2
        assert len(result.history) == 2
        assert result.history[0].log_id == "log_001"
        assert route.called


class TestUploadFileSign:
    """测试上传文件签名 API。"""

    @respx.mock
    def test_get_upload_file_sign_sync(self):
        """测试同步获取上传文件签名。"""
        client = XPayClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/get_upload_file_sign").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "sign": "abc123signature",
                    "cos_url": "https://cos.example.com/upload",
                },
            )
        )

        result = client.get_upload_file_sign(
            access_token="test_access_token",
            session_key="session_key_456",
            file_name="test.jpg",
            file_type="image/jpeg",
        )

        assert isinstance(result, models.UploadFileSign)
        assert result.sign == "abc123signature"
        assert result.cos_url == "https://cos.example.com/upload"
        assert route.called

    @respx.mock
    async def test_get_upload_file_sign_async(self):
        """测试异步获取上传文件签名。"""
        client = XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        route = respx.post("https://api.weixin.qq.com/xpay/get_upload_file_sign").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "sign": "abc123signature",
                    "cos_url": "https://cos.example.com/upload",
                },
            )
        )

        result = await client.get_upload_file_sign(
            access_token="test_access_token",
            session_key="session_key_456",
            file_name="test.png",
            file_type="image/png",
        )

        assert isinstance(result, models.UploadFileSign)
        assert result.sign == "abc123signature"
        assert route.called
        await client.close()



"""Tests for webhook message parser."""

import pytest

from wechat_xpay.webhook import (
    CoinPayNotify,
    ComplaintNotify,
    GoodsDeliverNotify,
    RefundNotify,
    WebhookParser,
)


class TestWebhookParser:
    """Test webhook parser."""

    def test_parse_goods_deliver_notify_json(self):
        """Test parsing goods delivery notification from JSON."""
        parser = WebhookParser()
        payload = {
            "ToUserName": "gh_123",
            "FromUserName": "official_openid",
            "CreateTime": 1700000000,
            "MsgType": "event",
            "Event": "xpay_goods_deliver_notify",
            "OpenId": "user_openid_123",
            "OutTradeNo": "order_123",
            "Env": 0,
            "WeChatPayInfo": {
                "MchOrderNo": "mch_order_123",
                "TransactionId": "transaction_456",
                "PaidTime": 1700000000,
            },
            "GoodsInfo": {
                "ProductId": "product_123",
                "Quantity": 1,
                "OrigPrice": 100,
                "ActualPrice": 100,
                "Attach": "custom_data",
            },
        }

        result = parser.parse(payload)

        assert isinstance(result, GoodsDeliverNotify)
        assert result.event == "xpay_goods_deliver_notify"
        assert result.open_id == "user_openid_123"
        assert result.out_trade_no == "order_123"
        assert result.env == 0
        assert result.wechat_pay_info is not None
        assert result.wechat_pay_info.transaction_id == "transaction_456"
        assert result.goods_info is not None
        assert result.goods_info.quantity == 1

    def test_parse_coin_pay_notify_json(self):
        """Test parsing coin payment notification from JSON."""
        parser = WebhookParser()
        payload = {
            "ToUserName": "gh_123",
            "FromUserName": "official_openid",
            "CreateTime": 1700000000,
            "MsgType": "event",
            "Event": "xpay_coin_pay_notify",
            "OpenId": "user_openid_123",
            "OutTradeNo": "order_123",
            "Env": 0,
            "CoinInfo": {
                "Quantity": 100,
                "OrigPrice": 100,
                "ActualPrice": 100,
                "Attach": "custom_data",
            },
        }

        result = parser.parse(payload)

        assert isinstance(result, CoinPayNotify)
        assert result.event == "xpay_coin_pay_notify"
        assert result.open_id == "user_openid_123"
        assert result.coin_info is not None
        assert result.coin_info.quantity == 100

    def test_parse_refund_notify_json(self):
        """Test parsing refund notification from JSON."""
        parser = WebhookParser()
        payload = {
            "ToUserName": "gh_123",
            "FromUserName": "official_openid",
            "CreateTime": 1700000000,
            "MsgType": "event",
            "Event": "xpay_refund_notify",
            "OpenId": "user_openid_123",
            "WxRefundId": "wx_refund_123",
            "MchRefundId": "mch_refund_456",
            "WxOrderId": "wx_order_789",
            "MchOrderId": "mch_order_abc",
            "RefundFee": 100,
            "RetCode": 0,
            "RetMsg": "Success",
            "RefundStartTimestamp": 1700000000,
            "RefundSuccTimestamp": 1700000100,
            "WxpayRefundTransactionId": "wxpay_refund_123",
            "RetryTimes": 0,
        }

        result = parser.parse(payload)

        assert isinstance(result, RefundNotify)
        assert result.event == "xpay_refund_notify"
        assert result.open_id == "user_openid_123"
        assert result.refund_fee == 100
        assert result.ret_code == 0

    def test_parse_complaint_notify_json(self):
        """Test parsing complaint notification from JSON."""
        parser = WebhookParser()
        payload = {
            "ToUserName": "gh_123",
            "FromUserName": "official_openid",
            "CreateTime": 1700000000,
            "MsgType": "event",
            "Event": "xpay_complaint_notify",
            "OpenId": "user_openid_123",
            "WxOrderId": "wx_order_123",
            "MchOrderId": "mch_order_456",
            "ComplaintTime": 1700000000,
            "RetryTimes": 0,
            "RequestId": "req_123",
        }

        result = parser.parse(payload)

        assert isinstance(result, ComplaintNotify)
        assert result.event == "xpay_complaint_notify"
        assert result.open_id == "user_openid_123"
        assert result.wx_order_id == "wx_order_123"

    def test_parse_unknown_event(self):
        """Test parsing unknown event type raises error."""
        parser = WebhookParser()
        payload = {
            "Event": "unknown_event",
        }

        with pytest.raises(ValueError) as exc_info:
            parser.parse(payload)

        assert "Unknown event type" in str(exc_info.value)

    def test_parse_xml(self):
        """Test parsing XML payload."""
        parser = WebhookParser()
        xml_payload = """<?xml version="1.0" encoding="UTF-8"?>
        <xml>
            <ToUserName>gh_123</ToUserName>
            <FromUserName>official_openid</FromUserName>
            <CreateTime>1700000000</CreateTime>
            <MsgType>event</MsgType>
            <Event>xpay_goods_deliver_notify</Event>
            <OpenId>user_openid_123</OpenId>
            <OutTradeNo>order_123</OutTradeNo>
            <Env>0</Env>
        </xml>"""

        result = parser.parse(xml_payload)

        assert isinstance(result, GoodsDeliverNotify)
        assert result.event == "xpay_goods_deliver_notify"
        assert result.open_id == "user_openid_123"



"""测试日志功能。"""

import logging
from io import StringIO

import pytest
import respx
from httpx import Response

from wechat_xpay import XPayAsyncClient, XPayClient
from wechat_xpay.exceptions import XPayAPIError


class TestLogging:
    """测试日志功能。"""

    def test_sync_client_with_logger(self):
        """测试同步客户端使用 logger。"""
        # 创建内存日志处理器
        log_stream = StringIO()
        handler = logging.StreamHandler(log_stream)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        handler.setFormatter(formatter)

        logger = logging.getLogger("test_sync")
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        with respx.mock:
            respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
                return_value=Response(
                    200,
                    json={
                        "errcode": 0,
                        "errmsg": "ok",
                        "balance": 1000,
                        "present_balance": 200,
                        "sum_save": 800,
                        "sum_present": 200,
                        "sum_balance": 1000,
                        "sum_cost": 500,
                        "first_save_flag": True,
                    },
                )
            )

            client = XPayClient(
                app_id="wx123",
                app_key="test_key",
                env=1,
                logger=logger,
            )

            client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="session_key_456",
            )

            client.close()

        # 验证日志内容
        log_output = log_stream.getvalue()
        assert "XPay API Request" in log_output
        assert "XPay API Response: Success" in log_output
        assert "/xpay/query_user_balance" in log_output

    def test_sync_client_without_logger(self):
        """测试同步客户端不使用 logger。"""
        with respx.mock:
            respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
                return_value=Response(
                    200,
                    json={
                        "errcode": 0,
                        "errmsg": "ok",
                        "balance": 1000,
                        "present_balance": 200,
                        "sum_save": 800,
                        "sum_present": 200,
                        "sum_balance": 1000,
                        "sum_cost": 500,
                        "first_save_flag": True,
                    },
                )
            )

            # 不传入 logger，应该正常工作
            client = XPayClient(
                app_id="wx123",
                app_key="test_key",
                env=1,
            )

            balance = client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="session_key_456",
            )

            assert balance.balance == 1000
            client.close()

    def test_sync_client_logs_error(self):
        """测试同步客户端记录错误日志。"""
        log_stream = StringIO()
        handler = logging.StreamHandler(log_stream)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        handler.setFormatter(formatter)

        logger = logging.getLogger("test_sync_error")
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        with respx.mock:
            respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
                return_value=Response(
                    200,
                    json={"errcode": -1, "errmsg": "系统错误"},
                )
            )

            client = XPayClient(
                app_id="wx123",
                app_key="test_key",
                env=1,
                logger=logger,
            )

            with pytest.raises(XPayAPIError):
                client.query_user_balance(
                    openid="user_123",
                    access_token="test_access_token",
                    session_key="session_key_456",
                )

            client.close()

        # 验证错误日志
        log_output = log_stream.getvalue()
        assert "ERROR" in log_output
        assert "XPay API Error" in log_output
        assert "系统错误" in log_output

    @respx.mock
    async def test_async_client_with_logger(self):
        """测试异步客户端使用 logger。"""
        log_stream = StringIO()
        handler = logging.StreamHandler(log_stream)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        handler.setFormatter(formatter)

        logger = logging.getLogger("test_async")
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance": 1000,
                    "present_balance": 200,
                    "sum_save": 800,
                    "sum_present": 200,
                    "sum_balance": 1000,
                    "sum_cost": 500,
                    "first_save_flag": True,
                },
            )
        )

        async with XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=1,
            logger=logger,
        ) as client:
            await client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="session_key_456",
            )

        # 验证日志内容
        log_output = log_stream.getvalue()
        assert "XPay API Request" in log_output
        assert "XPay API Response: Success" in log_output

    @respx.mock
    async def test_async_client_without_logger(self):
        """测试异步客户端不使用 logger。"""
        respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance": 1000,
                    "present_balance": 200,
                    "sum_save": 800,
                    "sum_present": 200,
                    "sum_balance": 1000,
                    "sum_cost": 500,
                    "first_save_flag": True,
                },
            )
        )

        # 不传入 logger，应该正常工作
        async with XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=1,
        ) as client:
            balance = await client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="session_key_456",
            )

            assert balance.balance == 1000



"""XPay 异步客户端。"""

from __future__ import annotations

import logging
from typing import Any

import httpx

from wechat_xpay import models
from wechat_xpay.base import BaseClient


class XPayAsyncClient(BaseClient):
    """XPay 异步客户端。

    使用 httpx.AsyncClient 进行异步 HTTP 请求。

    Args:
        app_id: 小程序 AppID
        app_key: 用于计算 pay_sig 的 AppKey
        env: 环境，0 表示沙箱，1 表示生产环境
        base_url: 可选的自定义基础 URL
        logger: 可选的日志记录器，用于记录 API 调用和响应信息

    Examples:
        # 方式 1：异步上下文管理器（推荐，自动关闭连接）
        async with XPayAsyncClient(
            app_id="wx1234567890",
            app_key="your_app_key",
            env=0,
        ) as client:
            balance = await client.query_user_balance(
                openid="user_openid",
                access_token="your_access_token",
                session_key="user_session_key",
            )
            print(f"余额: {balance.balance}")

        # 方式 2：手动管理生命周期
        client = XPayAsyncClient(
            app_id="wx1234567890",
            app_key="your_app_key",
            env=0,
        )
        balance = await client.query_user_balance(
            openid="user_openid",
            access_token="your_access_token",
            session_key="user_session_key",
        )
        print(f"余额: {balance.balance}")
        await client.close()

        # 方式 3：使用自定义 logger
        import logging
        logger = logging.getLogger("wechat_xpay")
        logger.setLevel(logging.DEBUG)
        handler = logging.StreamHandler()
        handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
        logger.addHandler(handler)

        async with XPayAsyncClient(
            app_id="wx1234567890",
            app_key="your_app_key",
            env=0,
            logger=logger,
        ) as client:
            balance = await client.query_user_balance(
                openid="user_openid",
                access_token="your_access_token",
                session_key="user_session_key",
            )
    """

    def __init__(
        self,
        app_id: str,
        app_key: str,
        env: int = 1,
        base_url: str | None = None,
        logger: logging.Logger | None = None,
    ) -> None:
        super().__init__(app_id, app_key, env, base_url, logger)
        self._client = httpx.AsyncClient()

    async def _http_post(
        self,
        endpoint: str,
        payload: dict[str, Any],
        access_token: str,
        session_key: str,
        *,
        needs_user_sig: bool = False,
    ) -> dict[str, Any]:
        """发送异步 HTTP POST 请求。

        Args:
            endpoint: API 端点路径
            payload: 请求体数据
            access_token: 接口调用凭证
            session_key: 用户的 session_key，用于计算用户态签名
            needs_user_sig: 是否需要用户态签名

        Returns:
            解析后的响应字典

        Raises:
            XPayAPIError: API 返回错误
            httpx.HTTPError: HTTP 请求错误
        """
        url, body_bytes, headers = self._prepare_request(
            endpoint,
            payload,
            access_token,
            session_key,
            needs_user_sig=needs_user_sig,
        )
        response = await self._client.post(url, content=body_bytes, headers=headers)
        response.raise_for_status()
        return self._handle_response(response.json())

    async def close(self) -> None:
        """关闭 HTTP 客户端连接池。

        释放与客户端关联的所有网络资源。
        """
        await self._client.aclose()

    async def __aenter__(self) -> XPayAsyncClient:
        """异步上下文管理器入口。

        Returns:
            XPayAsyncClient 实例
        """
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: object,
    ) -> None:
        """异步上下文管理器出口，自动关闭连接。

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常回溯
        """
        await self.close()

    # -------------------------------------------------------------------------
    # 用户代币管理 API
    # -------------------------------------------------------------------------

    async def query_user_balance(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        user_ip: str | None = None,
    ) -> models.UserBalance:
        """查询用户代币余额。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            user_ip: 用户 IP，例如 1.1.1.1

        Returns:
            UserBalance 对象，包含余额详情
        """
        payload: dict[str, Any] = {"openid": openid}
        if user_ip:
            payload["user_ip"] = user_ip
        response = await self._http_post(
            "/xpay/query_user_balance",
            payload,
            access_token,
            session_key,
            needs_user_sig=True,
        )
        return models.UserBalance(**response)

    async def currency_pay(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        order_id: str,
        amount: int,
        payitem: str,
        user_ip: str | None = None,
        remark: str | None = None,
    ) -> models.CurrencyPayResult:
        """扣除代币进行支付。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            order_id: 订单号
            amount: 支付的代币数量
            payitem: 物品信息，如 [{"productid":"物品id", "unit_price": 单价, "quantity": 数量}]
            user_ip: 用户 IP，例如 1.1.1.1
            remark: 备注

        Returns:
            CurrencyPayResult，包含订单 ID 和余额信息
        """
        payload: dict[str, Any] = {
            "openid": openid,
            "order_id": order_id,
            "amount": amount,
            "payitem": payitem,
        }
        if user_ip:
            payload["user_ip"] = user_ip
        if remark:
            payload["remark"] = remark
        response = await self._http_post(
            "/xpay/currency_pay",
            payload,
            access_token,
            session_key,
            needs_user_sig=True,
        )
        return models.CurrencyPayResult(**response)

    async def cancel_currency_pay(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        pay_order_id: str,
        order_id: str,
        amount: int,
        user_ip: str | None = None,
    ) -> models.CancelCurrencyPayResult:
        """取消代币支付（退款）。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            pay_order_id: 代币支付时传的 order_id
            order_id: 本次退款单的单号
            amount: 退款金额
            user_ip: 用户 IP，例如 1.1.1.1

        Returns:
            CancelCurrencyPayResult，包含退款订单 ID
        """
        payload: dict[str, Any] = {
            "openid": openid,
            "pay_order_id": pay_order_id,
            "order_id": order_id,
            "amount": amount,
        }
        if user_ip:
            payload["user_ip"] = user_ip
        response = await self._http_post(
            "/xpay/cancel_currency_pay", payload, access_token, session_key
        )
        return models.CancelCurrencyPayResult(**response)

    async def present_currency(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        order_id: str,
        amount: int,
    ) -> models.PresentCurrencyResult:
        """赠送代币给用户。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            order_id: 赠送订单 ID
            amount: 赠送金额

        Returns:
            PresentCurrencyResult，包含余额信息
        """
        payload: dict[str, Any] = {
            "openid": openid,
            "order_id": order_id,
            "amount": amount,
        }
        response = await self._http_post(
            "/xpay/present_currency", payload, access_token, session_key
        )
        return models.PresentCurrencyResult(**response)

    # -------------------------------------------------------------------------
    # 订单管理 API
    # -------------------------------------------------------------------------

    async def query_order(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        order_id: str | None = None,
        wx_order_id: str | None = None,
    ) -> models.Order:
        """查询订单详情。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            order_id: 创建的订单号（与 wx_order_id 二选一）
            wx_order_id: 微信内部单号（与 order_id 二选一）

        Returns:
            Order 对象，包含完整订单详情
        """
        payload: dict[str, Any] = {"openid": openid}
        if order_id:
            payload["order_id"] = order_id
        if wx_order_id:
            payload["wx_order_id"] = wx_order_id
        response = await self._http_post("/xpay/query_order", payload, access_token, session_key)
        return models.Order(**response)

    async def refund_order(
        self,
        openid: str,
        access_token: str,
        session_key: str,
        refund_order_id: str,
        left_fee: int,
        refund_fee: int,
        refund_reason: str,
        req_from: str,
        order_id: str | None = None,
        wx_order_id: str | None = None,
        biz_meta: str | None = None,
    ) -> models.RefundOrderResult:
        """现金订单退款。

        Args:
            openid: 用户的 OpenID
            session_key: 用户的 session_key，用于计算用户态签名
            refund_order_id: 退款订单 ID（8-32 位字符）
            left_fee: 当前可退金额
            refund_fee: 退款金额（0 < refund_fee <= left_fee）
            refund_reason: "0"-无, "1"-商品问题, "2"-服务, "3"-用户请求, "4"-价格, "5"-其他
            req_from: "1"-人工, "2"-用户发起, "3"-其他
            order_id: 下单时的单号（与 wx_order_id 二选一）
            wx_order_id: 支付单对应的微信侧单号（与 order_id 二选一）
            biz_meta: 自定义数据（0-1024 字符）

        Returns:
            RefundOrderResult，包含退款详情
        """
        payload: dict[str, Any] = {
            "openid": openid,
            "refund_order_id": refund_order_id,
            "left_fee": left_fee,
            "refund_fee": refund_fee,
            "refund_reason": refund_reason,
            "req_from": req_from,
        }
        if order_id:
            payload["order_id"] = order_id
        if wx_order_id:
            payload["wx_order_id"] = wx_order_id
        if biz_meta:
            payload["biz_meta"] = biz_meta
        response = await self._http_post("/xpay/refund_order", payload, access_token, session_key)
        return models.RefundOrderResult(**response)

    # -------------------------------------------------------------------------
    # 提现管理 API
    # -------------------------------------------------------------------------

    async def create_withdraw_order(
        self,
        access_token: str,
        session_key: str,
        withdraw_no: str,
        withdraw_amount: str | None = None,
    ) -> models.WithdrawOrderResult:
        """创建提现订单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            withdraw_no: 提现订单号（8-32 位字符）
            withdraw_amount: 提现金额（单位：元，如 "0.01"），省略表示全额提现

        Returns:
            WithdrawOrderResult，包含提现订单详情
        """
        payload: dict[str, Any] = {"withdraw_no": withdraw_no}
        if withdraw_amount:
            payload["withdraw_amount"] = withdraw_amount
        response = await self._http_post(
            "/xpay/create_withdraw_order", payload, access_token, session_key
        )
        return models.WithdrawOrderResult(**response)

    async def query_withdraw_order(
        self,
        access_token: str,
        session_key: str,
        withdraw_no: str,
    ) -> models.WithdrawOrder:
        """查询提现订单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            withdraw_no: 提现订单号

        Returns:
            WithdrawOrder，包含提现详情
        """
        payload = {"withdraw_no": withdraw_no}
        response = await self._http_post(
            "/xpay/query_withdraw_order", payload, access_token, session_key
        )
        return models.WithdrawOrder(**response)

    async def download_bill(
        self,
        access_token: str,
        session_key: str,
        begin_ds: int,
        end_ds: int,
    ) -> models.BillDownload:
        """下载小程序账单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            begin_ds: 开始日期（如 20230801）
            end_ds: 结束日期（如 20230810）

        Returns:
            BillDownload，包含下载 URL（30 分钟有效）
        """
        payload = {
            "begin_ds": begin_ds,
            "end_ds": end_ds,
        }
        response = await self._http_post("/xpay/download_bill", payload, access_token, session_key)
        return models.BillDownload(**response)

    # -------------------------------------------------------------------------
    # 商家余额和广告金 API
    # -------------------------------------------------------------------------

    async def query_biz_balance(
        self,
        access_token: str,
        session_key: str,
    ) -> models.BizBalance:
        """查询商家可提现余额。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名

        Returns:
            BizBalance，包含可用余额详情
        """
        payload: dict[str, Any] = {}
        response = await self._http_post(
            "/xpay/query_biz_balance", payload, access_token, session_key
        )
        balance_data = response.get("balance_available", {})
        return models.BizBalance(
            balance_available=models.BizBalanceAvailable(
                amount=balance_data.get("amount", "0"),
                currency_code=balance_data.get("currency_code", "CNY"),
            )
        )

    async def query_transfer_account(
        self,
        access_token: str,
        session_key: str,
    ) -> list:
        """查询广告金充值账户。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名

        Returns:
            TransferAccount 对象列表
        """
        payload: dict[str, Any] = {}
        response = await self._http_post(
            "/xpay/query_transfer_account", payload, access_token, session_key
        )
        return [models.TransferAccount(**acct) for acct in response.get("acct_list", [])]

    async def query_adver_funds(
        self,
        access_token: str,
        session_key: str,
        page: int = 1,
        page_size: int = 10,
    ) -> models.AdverFundList:
        """查询广告金发放记录。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            page: 页码（>= 1）
            page_size: 每页记录数

        Returns:
            AdverFundList，包含发放记录和分页信息
        """
        payload: dict[str, Any] = {
            "page": page,
            "page_size": page_size,
        }
        response = await self._http_post(
            "/xpay/query_adver_funds", payload, access_token, session_key
        )
        return models.AdverFundList(
            total_page=response.get("total_page", 1),
            adver_funds_list=[
                models.AdverFund(**fund) for fund in response.get("adver_funds_list", [])
            ],
        )

    # -------------------------------------------------------------------------
    # 投诉管理 API
    # -------------------------------------------------------------------------

    async def get_complaint_list(
        self,
        access_token: str,
        session_key: str,
        begin_date: str,
        end_date: str,
        offset: int = 0,
        limit: int = 10,
    ) -> models.ComplaintList:
        """获取投诉列表。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            begin_date: 开始日期（yyyy-mm-dd）
            end_date: 结束日期（yyyy-mm-dd）
            offset: 分页偏移量（从 0 开始）
            limit: 最大返回记录数

        Returns:
            ComplaintList，包含投诉列表和总数
        """
        payload: dict[str, Any] = {
            "begin_date": begin_date,
            "end_date": end_date,
            "offset": offset,
            "limit": limit,
        }
        response = await self._http_post(
            "/xpay/get_complaint_list", payload, access_token, session_key
        )
        return models.ComplaintList(
            total=response.get("total", 0),
            complaints=[models.Complaint(**c) for c in response.get("complaints", [])],
        )

    async def get_complaint_detail(
        self,
        access_token: str,
        session_key: str,
        complaint_id: str,
    ) -> models.Complaint:
        """获取投诉详情。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            complaint_id: 投诉 ID

        Returns:
            Complaint，包含完整详情
        """
        payload = {"complaint_id": complaint_id}
        response = await self._http_post(
            "/xpay/get_complaint_detail", payload, access_token, session_key
        )
        return models.Complaint(**response.get("complaint", {}))

    async def response_complaint(
        self,
        access_token: str,
        session_key: str,
        complaint_id: str,
        response_content: str,
        response_images: list | None = None,
    ) -> bool:
        """回复投诉。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            complaint_id: 投诉 ID
            response_content: 回复内容
            response_images: 图片文件 ID 列表（来自 upload_vp_file）

        Returns:
            成功返回 True
        """
        payload: dict[str, Any] = {
            "complaint_id": complaint_id,
            "response_content": response_content,
        }
        if response_images:
            payload["response_images"] = response_images
        await self._http_post("/xpay/response_complaint", payload, access_token, session_key)
        return True

    async def complete_complaint(
        self,
        access_token: str,
        session_key: str,
        complaint_id: str,
    ) -> bool:
        """完成投诉处理。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            complaint_id: 投诉 ID

        Returns:
            成功返回 True
        """
        payload = {"complaint_id": complaint_id}
        await self._http_post("/xpay/complete_complaint", payload, access_token, session_key)
        return True

    async def upload_vp_file(
        self,
        access_token: str,
        session_key: str,
        file_name: str,
        base64_img: str | None = None,
        img_url: str | None = None,
    ) -> models.UploadFileResult:
        """上传媒体文件（图片、凭证等）。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            file_name: 文件名称
            base64_img: Base64 编码的图片（最大 1MB）
            img_url: 图片 URL（最大 2MB，推荐）

        Returns:
            UploadFileResult，包含 file_id
        """
        payload: dict[str, Any] = {"file_name": file_name}
        if base64_img:
            payload["base64_img"] = base64_img
        if img_url:
            payload["img_url"] = img_url
        response = await self._http_post("/xpay/upload_vp_file", payload, access_token, session_key)
        return models.UploadFileResult(**response)

    # -------------------------------------------------------------------------
    # 发货管理 API
    # -------------------------------------------------------------------------

    async def notify_provide_goods(
        self,
        access_token: str,
        session_key: str,
        order_id: str | None = None,
        wx_order_id: str | None = None,
    ) -> models.NotifyProvideGoodsResult:
        """发货完成通知。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            order_id: 订单号（与 wx_order_id 二选一）
            wx_order_id: 微信内部单号（与 order_id 二选一）

        Returns:
            NotifyProvideGoodsResult，包含发货状态
        """
        payload: dict[str, Any] = {}
        if order_id:
            payload["order_id"] = order_id
        if wx_order_id:
            payload["wx_order_id"] = wx_order_id
        response = await self._http_post(
            "/xpay/notify_provide_goods", payload, access_token, session_key
        )
        return models.NotifyProvideGoodsResult(**response)

    # -------------------------------------------------------------------------
    # 道具管理 API
    # -------------------------------------------------------------------------

    async def start_upload_goods(
        self,
        access_token: str,
        session_key: str,
        goods: list[dict[str, Any]],
    ) -> models.GoodsUploadStatus:
        """启动道具上传。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            goods: 道具列表，每个道具包含 id, name, price, remark, item_url

        Returns:
            GoodsUploadStatus，包含上传任务状态
        """
        payload: dict[str, Any] = {"goods": goods}
        response = await self._http_post(
            "/xpay/start_upload_goods", payload, access_token, session_key
        )
        return models.GoodsUploadStatus(
            status=response.get("status", 0),
            upload_item=[
                models.GoodsUploadItem(**item) for item in response.get("upload_item", [])
            ],
        )

    async def query_upload_goods(
        self,
        access_token: str,
        session_key: str,
    ) -> models.GoodsUploadStatus:
        """查询道具上传状态。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名

        Returns:
            GoodsUploadStatus，包含上传任务状态和每个道具的上传状态
        """
        payload: dict[str, Any] = {}
        response = await self._http_post(
            "/xpay/query_upload_goods", payload, access_token, session_key
        )
        return models.GoodsUploadStatus(
            status=response.get("status", 0),
            upload_item=[
                models.GoodsUploadItem(**item) for item in response.get("upload_item", [])
            ],
        )

    async def start_publish_goods(
        self,
        access_token: str,
        session_key: str,
        goods: list[dict[str, Any]],
    ) -> models.GoodsPublishStatus:
        """启动道具发布。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            goods: 道具列表，每个道具包含 id

        Returns:
            GoodsPublishStatus，包含发布任务状态
        """
        payload: dict[str, Any] = {"goods": goods}
        response = await self._http_post(
            "/xpay/start_publish_goods", payload, access_token, session_key
        )
        return models.GoodsPublishStatus(
            status=response.get("status", 0),
            publish_item=[
                models.GoodsPublishItem(**item) for item in response.get("publish_item", [])
            ],
        )

    async def query_publish_goods(
        self,
        access_token: str,
        session_key: str,
    ) -> models.GoodsPublishStatus:
        """查询道具发布状态。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名

        Returns:
            GoodsPublishStatus，包含发布任务状态和每个道具的发布状态
        """
        payload: dict[str, Any] = {}
        response = await self._http_post(
            "/xpay/query_publish_goods", payload, access_token, session_key
        )
        return models.GoodsPublishStatus(
            status=response.get("status", 0),
            publish_item=[
                models.GoodsPublishItem(**item) for item in response.get("publish_item", [])
            ],
        )

    # -------------------------------------------------------------------------
    # 广告金账单 API
    # -------------------------------------------------------------------------

    async def create_funds_bill(
        self,
        access_token: str,
        session_key: str,
        transfer_account_uid: int,
        transfer_account_agency_id: int,
        transfer_amount: int,
        fund_id: str,
        settle_begin: int,
        settle_end: int,
        request_id: str | None = None,
    ) -> models.FundsBillResult:
        """创建广告金账单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            transfer_account_uid: 转账账户 UID
            transfer_account_agency_id: 转账账户代理 ID
            transfer_amount: 转账金额（单位：分）
            fund_id: 广告金发放记录 ID
            settle_begin: 结算开始时间戳
            settle_end: 结算结束时间戳
            request_id: 请求 ID（幂等控制，可选）

        Returns:
            FundsBillResult，包含账单 ID
        """
        payload: dict[str, Any] = {
            "transfer_account_uid": transfer_account_uid,
            "transfer_account_agency_id": transfer_account_agency_id,
            "transfer_amount": transfer_amount,
            "fund_id": fund_id,
            "settle_begin": settle_begin,
            "settle_end": settle_end,
        }
        if request_id:
            payload["request_id"] = request_id
        response = await self._http_post(
            "/xpay/create_funds_bill", payload, access_token, session_key
        )
        return models.FundsBillResult(**response)

    async def bind_transfer_account(
        self,
        access_token: str,
        session_key: str,
        transfer_account_uid: int,
        transfer_account_agency_id: int,
    ) -> bool:
        """绑定转账账户。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            transfer_account_uid: 转账账户 UID
            transfer_account_agency_id: 转账账户代理 ID

        Returns:
            成功返回 True
        """
        payload: dict[str, Any] = {
            "transfer_account_uid": transfer_account_uid,
            "transfer_account_agency_id": transfer_account_agency_id,
        }
        await self._http_post("/xpay/bind_transfer_accout", payload, access_token, session_key)
        return True

    async def query_funds_bill(
        self,
        access_token: str,
        session_key: str,
        page: int = 1,
        page_size: int = 10,
    ) -> models.FundsBillList:
        """查询资金账单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            page: 页码（>= 1）
            page_size: 每页记录数

        Returns:
            FundsBillList，包含账单列表和分页信息
        """
        payload: dict[str, Any] = {
            "page": page,
            "page_size": page_size,
        }
        response = await self._http_post(
            "/xpay/query_funds_bill", payload, access_token, session_key
        )
        return models.FundsBillList(
            total_page=response.get("total_page", 1),
            bill_list=[models.FundsBillItem(**item) for item in response.get("bill_list", [])],
        )

    async def query_recover_bill(
        self,
        access_token: str,
        session_key: str,
        page: int = 1,
        page_size: int = 10,
    ) -> models.RecoverBillList:
        """查询回收账单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            page: 页码（>= 1）
            page_size: 每页记录数

        Returns:
            RecoverBillList，包含回收账单列表和分页信息
        """
        payload: dict[str, Any] = {
            "page": page,
            "page_size": page_size,
        }
        response = await self._http_post(
            "/xpay/query_recover_bill", payload, access_token, session_key
        )
        return models.RecoverBillList(
            total_page=response.get("total_page", 1),
            bill_list=[models.RecoverBillItem(**item) for item in response.get("bill_list", [])],
        )

    async def download_adverfunds_order(
        self,
        access_token: str,
        session_key: str,
        begin_ds: int,
        end_ds: int,
    ) -> models.AdverfundsOrderDownload:
        """下载广告金订单账单。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            begin_ds: 开始日期（如 20230801）
            end_ds: 结束日期（如 20230810）

        Returns:
            AdverfundsOrderDownload，包含下载 URL
        """
        payload = {
            "begin_ds": begin_ds,
            "end_ds": end_ds,
        }
        response = await self._http_post(
            "/xpay/download_adverfunds_order", payload, access_token, session_key
        )
        return models.AdverfundsOrderDownload(**response)

    # -------------------------------------------------------------------------
    # 协商历史 API
    # -------------------------------------------------------------------------

    async def get_negotiation_history(
        self,
        access_token: str,
        session_key: str,
        complaint_id: str,
        offset: int = 0,
        limit: int = 10,
    ) -> models.NegotiationHistory:
        """获取协商历史。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            complaint_id: 投诉 ID
            offset: 分页偏移量（从 0 开始）
            limit: 最大返回记录数

        Returns:
            NegotiationHistory，包含协商记录列表
        """
        payload: dict[str, Any] = {
            "complaint_id": complaint_id,
            "offset": offset,
            "limit": limit,
        }
        response = await self._http_post(
            "/xpay/get_negotiation_history", payload, access_token, session_key
        )
        return models.NegotiationHistory(
            total=response.get("total", 0),
            history=[models.NegotiationRecord(**record) for record in response.get("history", [])],
        )

    # -------------------------------------------------------------------------
    # 文件上传签名 API
    # -------------------------------------------------------------------------

    async def get_upload_file_sign(
        self,
        access_token: str,
        session_key: str,
        file_name: str,
        file_type: str,
    ) -> models.UploadFileSign:
        """获取上传文件签名。

        Args:
            session_key: 用户的 session_key，用于计算用户态签名
            file_name: 文件名称
            file_type: 文件类型，如 "image/jpeg", "image/png"

        Returns:
            UploadFileSign，包含签名和上传 URL
        """
        payload: dict[str, Any] = {
            "file_name": file_name,
            "file_type": file_type,
        }
        response = await self._http_post(
            "/xpay/get_upload_file_sign", payload, access_token, session_key
        )
        return models.UploadFileSign(**response)



"""异步客户端测试。"""

import pytest
import respx
from httpx import Response

from wechat_xpay import XPayAsyncClient, models
from wechat_xpay.exceptions import XPayAPIError


class TestXPayAsyncClient:
    """测试 XPayAsyncClient 异步客户端。"""

    @pytest.fixture
    async def async_client(self):
        """提供异步客户端 fixture。"""
        client = XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        )
        yield client
        await client.close()

    @respx.mock
    async def test_query_user_balance(self, async_client):
        """测试异步查询用户余额。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance": 1000,
                    "present_balance": 200,
                    "sum_save": 800,
                    "sum_present": 200,
                    "sum_balance": 1000,
                    "sum_cost": 0,
                    "first_save_flag": True,
                },
            )
        )

        result = await async_client.query_user_balance(
            openid="user_123",
            access_token="test_access_token",
            session_key="session_key_456",
        )

        assert isinstance(result, models.UserBalance)
        assert result.balance == 1000
        assert result.present_balance == 200
        assert result.first_save_flag is True
        assert route.called

    @respx.mock
    async def test_currency_pay(self, async_client):
        """测试异步代币支付。"""
        route = respx.post("https://api.weixin.qq.com/xpay/currency_pay").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "order_123",
                    "balance": 800,
                    "used_present_amount": 100,
                },
            )
        )

        result = await async_client.currency_pay(
            openid="user_123",
            access_token="test_access_token",
            session_key="session_key_456",
            order_id="ORDER_001",
            amount=200,
            payitem="Test Item",
        )

        assert isinstance(result, models.CurrencyPayResult)
        assert result.order_id == "order_123"
        assert result.balance == 800
        assert route.called

    @respx.mock
    async def test_api_error(self, async_client):
        """测试异步 API 错误处理。"""
        respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 268490009,
                    "errmsg": "session_key 不存在或已过期",
                },
            )
        )

        with pytest.raises(XPayAPIError) as exc_info:
            await async_client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="expired_session_key",
            )

        assert exc_info.value.errcode == 268490009
        assert "session_key" in exc_info.value.errmsg

    @respx.mock
    async def test_http_error(self, async_client):
        """测试异步 HTTP 错误处理。"""
        respx.post("https://api.weixin.qq.com/xpay/query_user_balance").mock(
            return_value=Response(500, text="Internal Server Error")
        )

        with pytest.raises((XPayAPIError, Exception)):
            await async_client.query_user_balance(
                openid="user_123",
                access_token="test_access_token",
                session_key="session_key_456",
            )

    async def test_context_manager(self):
        """测试异步上下文管理器。"""
        async with XPayAsyncClient(
            app_id="wx123",
            app_key="test_key",
            env=0,
        ) as client:
            # 验证客户端已正确初始化
            assert client.app_id == "wx123"
            assert client._client is not None

    @respx.mock
    async def test_query_order(self, async_client):
        """测试异步查询订单。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "order_id": "order_123",
                    "create_time": 1700000000,
                    "update_time": 1700000100,
                    "status": 2,
                    "biz_type": 0,
                    "order_fee": 1000,
                    "paid_fee": 1000,
                    "order_type": 0,
                    "env_type": 1,
                },
            )
        )

        result = await async_client.query_order(
            openid="user_123",
            access_token="test_access_token",
            session_key="session_key_456",
            order_id="order_123",
        )

        assert isinstance(result, models.Order)
        assert result.order_id == "order_123"
        assert result.status == 2
        assert route.called

    @respx.mock
    async def test_refund_order(self, async_client):
        """测试异步退款。"""
        route = respx.post("https://api.weixin.qq.com/xpay/refund_order").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "refund_order_id": "refund_123",
                    "refund_wx_order_id": "wx_refund_456",
                    "pay_order_id": "order_123",
                    "pay_wx_order_id": "wx_order_456",
                },
            )
        )

        result = await async_client.refund_order(
            openid="user_123",
            access_token="test_access_token",
            session_key="session_key_456",
            refund_order_id="refund_123",
            left_fee=1000,
            refund_fee=500,
            refund_reason="1",
            req_from="1",
            order_id="original_order",
        )

        assert isinstance(result, models.RefundOrderResult)
        assert result.refund_order_id == "refund_123"
        assert route.called

    @respx.mock
    async def test_query_biz_balance(self, async_client):
        """测试异步查询商家余额。"""
        route = respx.post("https://api.weixin.qq.com/xpay/query_biz_balance").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "balance_available": {
                        "amount": "1000.00",
                        "currency_code": "CNY",
                    },
                },
            )
        )

        result = await async_client.query_biz_balance(
            access_token="test_access_token", session_key="session_key_456"
        )

        assert isinstance(result, models.BizBalance)
        assert result.balance_available.amount == "1000.00"
        assert route.called

    @respx.mock
    async def test_get_complaint_list(self, async_client):
        """测试异步获取投诉列表。"""
        route = respx.post("https://api.weixin.qq.com/xpay/get_complaint_list").mock(
            return_value=Response(
                200,
                json={
                    "errcode": 0,
                    "errmsg": "ok",
                    "total": 1,
                    "complaints": [
                        {
                            "complaint_id": "complaint_123",
                            "complaint_time": "2023-11-28T11:11:49+08:00",
                            "complaint_detail": "Test complaint",
                            "complaint_state": "PENDING",
                            "payer_openid": "openid_123",
                        }
                    ],
                },
            )
        )

        result = await async_client.get_complaint_list(
            access_token="test_access_token",
            session_key="session_key_456",
            begin_date="2023-01-01",
            end_date="2023-12-31",
            offset=0,
            limit=10,
        )

        assert isinstance(result, models.ComplaintList)
        assert result.total == 1
        assert len(result.complaints) == 1
        assert route.called



"""WeChat XPay (Virtual Payment) Python SDK."""

from wechat_xpay import models
from wechat_xpay.async_client import XPayAsyncClient
from wechat_xpay.auth import (
    calc_pay_sig,
    calc_signature,
    generate_nonce_str,
    generate_request_signature,
    generate_timestamp,
    verify_webhook_signature,
)
from wechat_xpay.client import XPayClient
from wechat_xpay.exceptions import (
    ERR_ACCOUNT_FROZEN,
    ERR_ADVER_FUNDS_NOT_ENOUGH,
    ERR_AMOUNT_MISMATCH,
    ERR_BAD_REQUEST_PARAMS,
    ERR_BALANCE_NOT_ENOUGH,
    ERR_BATCH_TASK_FAILED,
    ERR_BATCH_TASK_RUNNING,
    ERR_BILL_GENERATING,
    ERR_CANNOT_REFUND_VERIFIED_ORDER,
    ERR_COMPLAINT_NOT_EXIST,
    ERR_DATA_GENERATING,
    ERR_DUPLICATE_OPERATION,
    ERR_IMAGE_FORMAT_NOT_SUPPORTED,
    ERR_IMAGE_SIZE_EXCEEDED,
    ERR_INSUFFICIENT_TOKEN_BALANCE,
    ERR_INVALID_APPID,
    ERR_INVALID_BODY_PARAM,
    ERR_INVALID_ENV,
    ERR_INVALID_FILE_TYPE,
    ERR_INVALID_GOODS_CONFIG,
    ERR_INVALID_MEDIA_TYPE,
    ERR_INVALID_OPENID,
    ERR_INVALID_SIGN_TYPE,
    ERR_INVALID_TRANSACTION_ID,
    ERR_LEFT_FEE_MISMATCH,
    ERR_MCH_ACCOUNT_ABNORMAL,
    ERR_MCH_CONFIG_LIMITED,
    ERR_MCH_NO_PERMISSION,
    ERR_MCH_NOT_EXISTS,
    ERR_MEDIA_UPLOAD_FAILED,
    ERR_NEGOTIATION_FAILED,
    ERR_ORDER_ALREADY_REFUNDED,
    ERR_ORDER_CLOSED,
    ERR_ORDER_NOT_EXIST,
    ERR_ORDER_PAID,
    ERR_RATE_LIMIT_EXCEEDED,
    ERR_REAL_NAME_REQUIRED,
    ERR_REFUND_AMOUNT_LIMIT,
    ERR_REFUND_APPLICATION_EXIST,
    ERR_REFUND_FEE_EXCEEDED,
    ERR_REFUND_FEE_LIMIT,
    ERR_REFUND_IN_PROGRESS,
    ERR_REFUND_NOT_EXIST,
    ERR_SENSITIVE_CONTENT,
    ERR_SERVICE_NOT_ENABLED,
    ERR_SESSION_KEY_EXPIRED,
    ERR_SIGN_VERIFICATION_FAILED,
    ERR_SIGNATURE_ERROR,
    ERR_SYSTEM_BUSY,
    ERR_SYSTEM_ERROR,
    ERR_TOKEN_NOT_PUBLISHED,
    ERR_TRANSFER_ACCOUNT_NOT_BOUND,
    ERR_UPLOAD_FILE_FAILED,
    ERR_USER_NOT_REGISTERED,
    ERR_VIDEO_FORMAT_NOT_SUPPORTED,
    ERR_VIDEO_SIZE_EXCEEDED,
    ERR_WITHDRAW_AMOUNT_LIMIT,
    ERR_WITHDRAW_COUNT_LIMIT,
    ERR_WITHDRAW_FEE_LIMIT,
    XPayAPIError,
    XPayError,
)

__all__ = [
    # Client
    "XPayClient",
    "XPayAsyncClient",
    # Auth
    "calc_pay_sig",
    "calc_signature",
    "generate_request_signature",
    "verify_webhook_signature",
    "generate_timestamp",
    "generate_nonce_str",
    # Exceptions
    "XPayError",
    "XPayAPIError",
    # Error Codes
    "ERR_SYSTEM_ERROR",
    "ERR_INVALID_OPENID",
    "ERR_BAD_REQUEST_PARAMS",
    "ERR_SIGNATURE_ERROR",
    "ERR_DUPLICATE_OPERATION",
    "ERR_ORDER_ALREADY_REFUNDED",
    "ERR_INSUFFICIENT_TOKEN_BALANCE",
    "ERR_SENSITIVE_CONTENT",
    "ERR_TOKEN_NOT_PUBLISHED",
    "ERR_SESSION_KEY_EXPIRED",
    "ERR_DATA_GENERATING",
    "ERR_BATCH_TASK_RUNNING",
    "ERR_CANNOT_REFUND_VERIFIED_ORDER",
    "ERR_REFUND_IN_PROGRESS",
    "ERR_RATE_LIMIT_EXCEEDED",
    "ERR_LEFT_FEE_MISMATCH",
    "ERR_ACCOUNT_FROZEN",
    "ERR_ADVER_FUNDS_NOT_ENOUGH",
    "ERR_AMOUNT_MISMATCH",
    "ERR_BALANCE_NOT_ENOUGH",
    "ERR_BATCH_TASK_FAILED",
    "ERR_BILL_GENERATING",
    "ERR_COMPLAINT_NOT_EXIST",
    "ERR_IMAGE_FORMAT_NOT_SUPPORTED",
    "ERR_IMAGE_SIZE_EXCEEDED",
    "ERR_INVALID_APPID",
    "ERR_INVALID_BODY_PARAM",
    "ERR_INVALID_ENV",
    "ERR_INVALID_FILE_TYPE",
    "ERR_INVALID_GOODS_CONFIG",
    "ERR_INVALID_MEDIA_TYPE",
    "ERR_INVALID_SIGN_TYPE",
    "ERR_INVALID_TRANSACTION_ID",
    "ERR_MCH_ACCOUNT_ABNORMAL",
    "ERR_MCH_CONFIG_LIMITED",
    "ERR_MCH_NO_PERMISSION",
    "ERR_MCH_NOT_EXISTS",
    "ERR_MEDIA_UPLOAD_FAILED",
    "ERR_NEGOTIATION_FAILED",
    "ERR_ORDER_CLOSED",
    "ERR_ORDER_NOT_EXIST",
    "ERR_ORDER_PAID",
    "ERR_REAL_NAME_REQUIRED",
    "ERR_REFUND_AMOUNT_LIMIT",
    "ERR_REFUND_APPLICATION_EXIST",
    "ERR_REFUND_FEE_EXCEEDED",
    "ERR_REFUND_FEE_LIMIT",
    "ERR_REFUND_NOT_EXIST",
    "ERR_SERVICE_NOT_ENABLED",
    "ERR_SIGN_VERIFICATION_FAILED",
    "ERR_SYSTEM_BUSY",
    "ERR_TRANSFER_ACCOUNT_NOT_BOUND",
    "ERR_UPLOAD_FILE_FAILED",
    "ERR_USER_NOT_REGISTERED",
    "ERR_VIDEO_FORMAT_NOT_SUPPORTED",
    "ERR_VIDEO_SIZE_EXCEEDED",
    "ERR_WITHDRAW_AMOUNT_LIMIT",
    "ERR_WITHDRAW_COUNT_LIMIT",
    "ERR_WITHDRAW_FEE_LIMIT",
    # Models
    "models",
]



"""
Signature computation for WeChat XPay server API.

Two independent HMAC-SHA256 signatures:
  pay_sig   - keyed by appKey,     message = uri + '&' + signData
  signature - keyed by session_key, message = signData
"""

import hashlib
import hmac


def calc_pay_sig(uri: str, signData: str, appkey: str) -> str:  # noqa: N803
    """pay_sig签名算法
    Args:
       uri      - 前端wx API填"requestVirtualPayment"或后端填当前请求的API的uri部分，不带query_string 例如："/xpay/query_user_balance"
       signData - 前端wx API的signData字段或后端http POST的数据包体
       appkey   - 对应环境的AppKey
    Returns:
       支付请求签名pay_sig
    """
    need_sign_msg = uri + "&" + signData
    pay_sig = hmac.new(
        key=appkey.encode("utf-8"), msg=need_sign_msg.encode("utf-8"), digestmod=hashlib.sha256
    ).hexdigest()
    return pay_sig


def calc_signature(signData: str, session_key: str) -> str:  # noqa: N803
    """用户登录态signature签名算法
    Args:
        signData    - 前端wx API的signData字段或后端http POST的数据包体
        session_key - 当前用户有效的session_key，参考auth.code2Session接口
    Returns:
        用户登录态签名signature
    """
    need_sign_msg = signData
    signature = hmac.new(
        key=session_key.encode("utf-8"), msg=need_sign_msg.encode("utf-8"), digestmod=hashlib.sha256
    ).hexdigest()
    return signature


def generate_request_signature(
    uri: str,
    payload: dict,
    app_key: str,
    session_key: str,
) -> tuple[str, str]:
    """Generate both signatures for a request.

    This is a convenience function that generates both pay_sig and
    signature for a complete request payload.

    Args:
        uri:          API endpoint path (e.g., '/xpay/query_user_balance')
        payload:      Request body as a dictionary
        app_key:      AppKey for pay_sig calculation
        session_key:  User's session_key for signature calculation

    Returns:
        Tuple of (pay_sig, signature)
    """
    import json

    body_str = json.dumps(payload, ensure_ascii=False)
    pay_sig = calc_pay_sig(uri, body_str, app_key)
    signature = calc_signature(body_str, session_key)
    return pay_sig, signature


def verify_webhook_signature(
    body: str,
    signature: str,
    session_key: str,
) -> bool:
    """Verify webhook callback signature.

    Args:
        body:          Raw request body string
        signature:     Signature from X-Signature header
        session_key:   User's session_key used to sign the request

    Returns:
        True if signature is valid, False otherwise
    """
    expected = calc_signature(body, session_key)
    return hmac.compare_digest(expected, signature)


def generate_timestamp() -> int:
    """Generate current Unix timestamp in seconds.

    Returns:
        Current Unix timestamp
    """
    import time

    return int(time.time())


def generate_nonce_str(length: int = 32) -> str:
    """Generate a random nonce string.

    Args:
        length: Length of the nonce string (default: 32)

    Returns:
        Random alphanumeric string
    """
    import secrets

    return secrets.token_hex(length // 2)
